package dk.nsp.backgroundjob.service;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.backgroundjob.util.DateTimeProvider;
import org.apache.log4j.Logger;

import java.time.Duration;
import java.time.ZonedDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Stack;
import java.util.stream.Collectors;

public class StackedOperationsService {
    private final Logger log = Logger.getLogger(StackedOperationsService.class);

    private final StackOperation defaultStackOperation;
    private final Duration desiredExecutionDuration;
    private final DateTimeProvider dateTimeProvider;
    private final Stack<StackOperation> stack;
    private String serviceName = "Operations service";
    private boolean latestRunSuccessful = true;

    public StackedOperationsService(StackOperation defaultStackOperation, Duration desiredExecutionDuration, DateTimeProvider dateTimeProvider) {
        this.defaultStackOperation = defaultStackOperation;
        this.desiredExecutionDuration = desiredExecutionDuration;
        this.dateTimeProvider = dateTimeProvider;
        stack = new Stack<>();
    }

    public StackedOperationsService withServiceName(String serviceName) {
        this.serviceName = serviceName;
        return this;
    }

    public void run() {
        ZonedDateTime startTime = dateTimeProvider.now();
        try {
            synchronizedRun(startTime);
            latestRunSuccessful = true;
        } catch (Exception e) {
            latestRunSuccessful = false;
            throw e;
        }
    }

    public boolean latestRunSuccessful() {
        return latestRunSuccessful;
    }

    private synchronized void synchronizedRun(ZonedDateTime startTime) {
        log.trace(String.format("%s: Execution started.", serviceName));
        if (!desiredExecutionDurationReached(startTime)) {
            fillStackIfEmpty();
        }

        try {
            while(!desiredExecutionDurationReached(startTime) && !stack.isEmpty()) {
                runOperationAndPopulateStack(stack.pop());
            }
        } finally {
            logRemaining();
            log.info(String.format("%s: Execution finished. Time spent: %d seconds", serviceName, Duration.between(startTime, dateTimeProvider.now()).getSeconds()));
        }
    }

    private void fillStackIfEmpty() {
        if (stack.isEmpty()) {
            log.trace(serviceName + ": Stack is empty");
            runOperationAndPopulateStack(defaultStackOperation);
        }
    }

    private void runOperationAndPopulateStack(StackOperation stackOperation) {
        List<StackOperation> newStackOperations = stackOperation.run();
        Collections.shuffle(newStackOperations);
        newStackOperations.forEach(stack::push);
        log.info(String.format("%s: %d new operations added to stack from %s", serviceName, newStackOperations.size(), stackOperation.getClass().getSimpleName()));
    }

    private boolean desiredExecutionDurationReached(ZonedDateTime startTime) {
        return Duration.between(startTime, dateTimeProvider.now()).compareTo(desiredExecutionDuration) > 0;
    }

    private void logRemaining() {
        log.info(String.format("%s: remaining=%d on stack", serviceName, stack.size()));
        stack.stream().collect(Collectors.groupingBy(stackOperation -> stackOperation.getClass().getSimpleName(), Collectors.counting()))
                .forEach((operationName, count) ->
                        log.info(String.format("%s: remaining=%d of type %s on stack", serviceName, count, operationName)));
    }
}
