package dk.nsp.xdscleanup.setup;

import dk.nsp.backgroundjob.service.StackedOperationsService;
import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.model.DocumentTypeCleanupSpecification;
import dk.nsp.xdscleanup.nxrg.dao.NxrgDao;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.openxds.dao.OpenXdsDao;
import dk.nsp.xdscleanup.openxds.service.RepositoryDeletionService;
import dk.nsp.xdscleanup.sdm.dao.SdmDao;
import dk.nsp.xdscleanup.stack.registry.status90.Status90RegistryCleanupSupplier;
import dk.nsp.xdscleanup.stack.registry.timebased.TypeSpecificationBasedRegistryCleanupSupplier;
import dk.nsp.xdscleanup.stack.repository.RepositoryCleanupSupplier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.support.PropertySourcesPlaceholderConfigurer;

import java.time.Duration;
import java.time.Period;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Configuration
@PropertySource("classpath:xdscleanup.properties")
@Import({DBSetup.class, VersionSetup.class})
public class XdsCleanupSetup {
    @Value("${deletion.registry.batchsize}")
    private int batchSizeRegistry;

    @Value("${deletion.repository.batchsize}")
    private int batchSizeRepository;

    @Value("${deletion.registry.timebased.limit}")
    private int registryTimebasedLimit;

    @Value("${deletion.repository.limit}")
    private int repositoryLimit;

    @Value("${deletion.save.status90.docs}")
    private String deletionSaveStatus90Docs;

    @Value("${desired.execution.duration}")
    private String desiredExecutionDuration;

    @Bean
    public static PropertySourcesPlaceholderConfigurer propertySourcesPlaceholderConfigurer() {
        return new PropertySourcesPlaceholderConfigurer();
    }

    @Bean
    public RepositoryDeletionService repositoryDeletionService(OpenXdsDao openXdsDao) {
        return new RepositoryDeletionService(openXdsDao);
    }

    @Bean
    public RegistryDeletionService registryDeletionService(NxrgDao nxrgDao) {
        return new RegistryDeletionService(nxrgDao);
    }

    @Bean
    public StackedOperationsService registryTimeBasedCleanupService(List<DocumentTypeCleanupSpecification> documentTypeCleanupSpecifications,
                                                                          CleanupOperationsFactory cleanupOperationsFactory, Duration desiredExecutionDuration,
                                                                          DateTimeProvider dateTimeProvider) {

        TypeSpecificationBasedRegistryCleanupSupplier defaultStackOperation = new TypeSpecificationBasedRegistryCleanupSupplier(documentTypeCleanupSpecifications, cleanupOperationsFactory);
        return new StackedOperationsService(defaultStackOperation, desiredExecutionDuration, dateTimeProvider).withServiceName("Registry Time Based Cleanup");
    }

    @Bean
    public StackedOperationsService registryStatus90CleanupService(CleanupOperationsFactory cleanupOperationsFactory, Duration desiredExecutionDuration,
                                                                         DateTimeProvider dateTimeProvider) {
        Status90RegistryCleanupSupplier defaultStackOperation = new Status90RegistryCleanupSupplier(cleanupOperationsFactory);
        return new StackedOperationsService(defaultStackOperation, desiredExecutionDuration, dateTimeProvider).withServiceName("Registry Status 90 Cleanup");
    }

    @Bean
    public StackedOperationsService repositoryCleanupService(RegistryDeletionService registryDeletionService, CleanupOperationsFactory cleanupOperationsFactory,
                                                                   Duration desiredExecutionDuration, DateTimeProvider dateTimeProvider) {
        RepositoryCleanupSupplier defaultStackOperation = new RepositoryCleanupSupplier(registryDeletionService, cleanupOperationsFactory, batchSizeRepository, repositoryLimit);
        return new StackedOperationsService(defaultStackOperation, desiredExecutionDuration, dateTimeProvider).withServiceName("Repository Cleanup");
    }

    @Bean
    public CleanupOperationsFactory cleanupOperationsFactory(RegistryDeletionService registryDeletionService, RepositoryDeletionService repositoryDeletionService,
                                                             SdmDao sdmDao, DateTimeProvider dateTimeProvider, Period status90Period) {
        return new CleanupOperationsFactory(registryDeletionService, repositoryDeletionService, sdmDao, dateTimeProvider, batchSizeRegistry, status90Period, registryTimebasedLimit);
    }

    @Bean
    public DateTimeProvider dateTimeProvider() {
        return new DateTimeProvider();
    }

    @Bean
    public List<DocumentTypeCleanupSpecification> getDocumentTypeCleanupSpecifications(@Value("${deletion.document.cleanup.specifications}") String[] cleanupSpecifications) {
        return Arrays.stream(cleanupSpecifications)
                .map(DocumentTypeCleanupSpecification::parse)
                .collect(Collectors.toList());
    }

    @Bean
    public Period status90Period() {
        return Period.parse("P" + deletionSaveStatus90Docs);
    }

    @Bean
    public Duration desiredExecutionDuration() {
        return Duration.parse(desiredExecutionDuration);
    }
}
