package dk.nsp.xdscleanup.stack.registry.timebased;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.common.util.CleanupUtils;
import dk.nsp.xdscleanup.common.util.DateTimeUtils;
import dk.nsp.xdscleanup.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.model.DocumentTypeCleanupSpecification;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.common.util.CleanupException;

import java.time.Instant;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

public class TypeAndTimeBasedRegistryCleanupSupplier implements StackOperation {
    private final DocumentTypeCleanupSpecification typeSpecification;
    private final RegistryDeletionService registryDeletionService;
    private final DateTimeProvider dateTimeProvider;
    private final int batchSize;
    private final CleanupOperationsFactory cleanupOperationsFactory;
    private final int limit;

    public TypeAndTimeBasedRegistryCleanupSupplier(DocumentTypeCleanupSpecification typeSpecification,
                                                   RegistryDeletionService registryDeletionService,
                                                   DateTimeProvider dateTimeProvider, int batchSize,
                                                   CleanupOperationsFactory cleanupOperationsFactory,
                                                   int limit) {
        this.typeSpecification = typeSpecification;
        this.registryDeletionService = registryDeletionService;
        this.dateTimeProvider = dateTimeProvider;
        this.batchSize = batchSize;
        this.cleanupOperationsFactory = cleanupOperationsFactory;
        this.limit = limit;
    }

    @Override
    public List<StackOperation> run() {
        Instant latestStartTime = computeLatestStartTime(typeSpecification.getNumberOfMonthsToKeep());
        List<DocumentEntry> documentEntriesForDeletion = getDocumentEntriesForDeletion(latestStartTime);
        Collection<List<DocumentEntry>> entryBatches = CleanupUtils.createShuffledBatches(documentEntriesForDeletion, batchSize);

        return entryBatches.stream()
                .map(batch -> cleanupOperationsFactory.createRegistryBatchCleanupOperation(batch).withOperationName("Registry Time Based Cleanup"))
                .collect(Collectors.toList());
    }

    private List<DocumentEntry> getDocumentEntriesForDeletion(Instant latestStartTime) {
        try {
            return registryDeletionService.getDocumentEntriesForDeletion(typeSpecification.getTypeCode(), latestStartTime, limit);
        } catch (Exception e) {
            throw new CleanupException(String.format("Failed to retrieve document entries with latestStartTime %s and type code %s", latestStartTime, typeSpecification.getTypeCode()), e);
        }
    }

    private Instant computeLatestStartTime(int monthsToKeep) {
        return DateTimeUtils.subtractMonths(dateTimeProvider.now(), monthsToKeep).toInstant();
    }

}
