package dk.nsp.xdscleanup.stack.repository;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.openxds.service.RepositoryDeletionService;
import dk.nsp.xdscleanup.common.util.CleanupException;
import org.apache.log4j.Logger;

import java.time.Duration;
import java.time.ZonedDateTime;
import java.util.Collections;
import java.util.List;

public class RepositoryBatchCleanupOperation implements StackOperation {
    private static final String OPERATION_NAME = "Repository Cleanup";
    private final Logger log = Logger.getLogger(RepositoryBatchCleanupOperation.class);
    private final List<DocumentEntry> documentEntriesForDeletion;
    private final DateTimeProvider dateTimeProvider;
    private final RepositoryDeletionService repositoryDeletionService;
    private final RegistryDeletionService registryDeletionService;

    public RepositoryBatchCleanupOperation(List<DocumentEntry> documentEntriesForDeletion, DateTimeProvider dateTimeProvider,
                                           RepositoryDeletionService repositoryDeletionService, RegistryDeletionService registryDeletionService) {
        this.documentEntriesForDeletion = documentEntriesForDeletion;
        this.dateTimeProvider = dateTimeProvider;
        this.repositoryDeletionService = repositoryDeletionService;
        this.registryDeletionService = registryDeletionService;
    }

    @Override
    public List<StackOperation> run() {
        log.trace(String.format("%s: Starting batch deletion of %d entries.", OPERATION_NAME, documentEntriesForDeletion.size()));
        ZonedDateTime startTime = dateTimeProvider.now();

        List<DocumentEntry> deletedDocuments;
        try {
            deletedDocuments = repositoryDeletionService.deleteDocuments(documentEntriesForDeletion);
        } catch (Exception e) {
            markDeletionAttemptFailed();
            throw new CleanupException("Failed to delete documents " + documentEntriesForDeletion, e);
        }

        log.info(String.format("%s: Batch cleanup finished! deleted=%d Time spent: %d seconds.", OPERATION_NAME, deletedDocuments.size(), Duration.between(startTime, dateTimeProvider.now()).getSeconds()));
        log.info(String.format("%s: Deleted documents are: %s", OPERATION_NAME, deletedDocuments));
        markDeletionComplete();
        return Collections.emptyList();
    }

    private void markDeletionComplete() {
        try {
            registryDeletionService.markDeletionComplete(documentEntriesForDeletion);
            log.trace(String.format("%s: Batch marked as completed", OPERATION_NAME));
        } catch (Exception e) {
            log.error(String.format("%s: Failed to mark successful repository deletion of document entries %s", OPERATION_NAME, documentEntriesForDeletion), e);
        }
    }

    private void markDeletionAttemptFailed() {
        try {
            registryDeletionService.markDeletionAttemptFailed(documentEntriesForDeletion);
            log.trace(String.format("%s: Batch marked as failed", OPERATION_NAME));
        } catch (Exception e) {
            log.error(String.format("%s: Failed to mark failed repository deletion of document entries %s", OPERATION_NAME, documentEntriesForDeletion), e);
        }
    }
}
