package dk.nsp.xdscleanup.stack.registry.status90;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.sdm.dao.SdmDao;
import dk.nsp.xdscleanup.stack.registry.RegistryBatchCleanupOperation;
import dk.nsp.xdscleanup.common.util.CleanupException;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.time.Period;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static dk.nsp.xdscleanup.TestUtils.assertThrows;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class Status90RegistryPatientIdBatchCleanupSupplierTest {

    private RegistryDeletionService registryDeletionService;
    private DateTimeProvider dateTimeProvider;
    private CleanupOperationsFactory cleanupOperationsFactory;
    private SdmDao sdmDao;

    @Before
    public void setUp() throws Exception {
        registryDeletionService = Mockito.mock(RegistryDeletionService.class);
        sdmDao = Mockito.mock(SdmDao.class);
        dateTimeProvider = Mockito.mock(DateTimeProvider.class);
        cleanupOperationsFactory = Mockito.mock(CleanupOperationsFactory.class);
    }

    @Test
    public void run_WithNoDocumentsForCitizens_ReturnsEmptyList() {
        List<String> citizenCprs = Collections.singletonList("1234567890");
        Status90RegistryPatientIdBatchCleanupSupplier status90RegistryPatientIdBatchCleanupSupplier = new Status90RegistryPatientIdBatchCleanupSupplier(
                citizenCprs, registryDeletionService, sdmDao, dateTimeProvider, cleanupOperationsFactory, 1, Period.ofMonths(1));
        Mockito.when(dateTimeProvider.now()).thenReturn(ZonedDateTime.now());
        Mockito.when(registryDeletionService.getDocumentEntriesForCitizens(Mockito.anyList())).thenReturn(Collections.emptyList());

        List<StackOperation> stackOperations = status90RegistryPatientIdBatchCleanupSupplier.run();

        assertEquals(0, stackOperations.size());
    }

    @Test
    public void run_With3DocumentsForCitizensBatchSize1_Returns3StackOperationWithDocumentEntries() {
        List<String> citizenCprs = Arrays.asList("0000000000", "1111111111", "2222222222");
        Period status90Period = Period.ofMonths(1);
        Status90RegistryPatientIdBatchCleanupSupplier status90RegistryPatientIdBatchCleanupSupplier = new Status90RegistryPatientIdBatchCleanupSupplier(
                citizenCprs, registryDeletionService, sdmDao, dateTimeProvider, cleanupOperationsFactory, 1, status90Period);
        ZonedDateTime now = ZonedDateTime.now();
        Mockito.when(dateTimeProvider.now()).thenReturn(now);
        List<String> status90Cprs = Arrays.asList("1111111111", "2222222222");
        Mockito.when(sdmDao.getStatus90Citizens(citizenCprs, now.minus(status90Period).toInstant()))
                .thenReturn(status90Cprs);
        DocumentEntry entry1 = Mockito.mock(DocumentEntry.class);
        DocumentEntry entry2 = Mockito.mock(DocumentEntry.class);
        DocumentEntry entry3 = Mockito.mock(DocumentEntry.class);
        List<DocumentEntry> documentEntries = Arrays.asList(entry1, entry2, entry3);
        Mockito.when(registryDeletionService.getDocumentEntriesForCitizens(status90Cprs)).thenReturn(documentEntries);
        RegistryBatchCleanupOperation batchOperation1 = Mockito.mock(RegistryBatchCleanupOperation.class);
        Mockito.when(batchOperation1.withOperationName(Mockito.anyString())).thenReturn(batchOperation1);
        Mockito.when(cleanupOperationsFactory.createRegistryBatchCleanupOperation(Arrays.asList(entry1))).thenReturn(batchOperation1);
        RegistryBatchCleanupOperation batchOperation2 = Mockito.mock(RegistryBatchCleanupOperation.class);
        Mockito.when(batchOperation2.withOperationName(Mockito.anyString())).thenReturn(batchOperation2);
        Mockito.when(cleanupOperationsFactory.createRegistryBatchCleanupOperation(Arrays.asList(entry2))).thenReturn(batchOperation2);
        RegistryBatchCleanupOperation batchOperation3 = Mockito.mock(RegistryBatchCleanupOperation.class);
        Mockito.when(batchOperation3.withOperationName(Mockito.anyString())).thenReturn(batchOperation3);
        Mockito.when(cleanupOperationsFactory.createRegistryBatchCleanupOperation(Arrays.asList(entry3))).thenReturn(batchOperation3);

        List<StackOperation> stackOperations = status90RegistryPatientIdBatchCleanupSupplier.run();

        assertEquals(3, stackOperations.size());
        assertTrue(stackOperations.contains(batchOperation1));
        assertTrue(stackOperations.contains(batchOperation2));
        assertTrue(stackOperations.contains(batchOperation3));
    }

    @Test
    public void run_WhenRetrievingStatus90CitizensFails_ThrowsCleanupException() {
        Mockito.when(dateTimeProvider.now()).thenReturn(ZonedDateTime.now());
        Mockito.when(sdmDao.getStatus90Citizens(Mockito.anyList(), Mockito.any())).thenThrow(RuntimeException.class);
        Status90RegistryPatientIdBatchCleanupSupplier status90RegistryPatientIdBatchCleanupSupplier = new Status90RegistryPatientIdBatchCleanupSupplier(
                Collections.emptyList(), registryDeletionService, sdmDao, dateTimeProvider, cleanupOperationsFactory, 1, Period.ofMonths(1));

        CleanupException e = assertThrows(CleanupException.class, () -> status90RegistryPatientIdBatchCleanupSupplier.run());

        assertTrue(e.getMessage(), e.getMessage().contains("Failed to retrieve status 90 citizens with latestStatusTime"));
    }

    @Test
    public void run_WhenRetrievingDocumentEntriesFails_ThrowsCleanupException() {
        Mockito.when(dateTimeProvider.now()).thenReturn(ZonedDateTime.now());
        Mockito.when(registryDeletionService.getDocumentEntriesForCitizens(Mockito.anyList())).thenThrow(RuntimeException.class);
        Status90RegistryPatientIdBatchCleanupSupplier status90RegistryPatientIdBatchCleanupSupplier = new Status90RegistryPatientIdBatchCleanupSupplier(
                Collections.emptyList(), registryDeletionService, sdmDao, dateTimeProvider, cleanupOperationsFactory, 1, Period.ofMonths(1));

        CleanupException e = assertThrows(CleanupException.class, () -> status90RegistryPatientIdBatchCleanupSupplier.run());

        assertEquals("Failed to retrieve document entries for citizens", e.getMessage());
    }
}