package dk.nsp.xdscleanup.stack.registry.status90;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.xdscleanup.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.common.util.CleanupException;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.ArgumentMatcher;
import org.mockito.Mockito;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import static dk.nsp.xdscleanup.TestUtils.assertThrows;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class Status90RegistryPatientIdPrefixCleanupSupplierTest {

    private RegistryDeletionService registryDeletionService;
    private CleanupOperationsFactory cleanupOperationsFactory;

    @Before
    public void setUp() {
        registryDeletionService = Mockito.mock(RegistryDeletionService.class);
        cleanupOperationsFactory = Mockito.mock(CleanupOperationsFactory.class);
    }

    @Test
    public void run_WithNoPatientIdsForPrefix_ReturnsEmptyList() {
        Status90RegistryPatientIdPrefixCleanupSupplier status90RegistryPatientIdPrefixCleanupSupplier =
                new Status90RegistryPatientIdPrefixCleanupSupplier(registryDeletionService, cleanupOperationsFactory, 5, "01");
        Mockito.when(registryDeletionService.getDocumentEntryPatientIdsWithPrefix(Mockito.anyString()))
                .thenReturn(Collections.emptyList());

        List<StackOperation> stackOperations = status90RegistryPatientIdPrefixCleanupSupplier.run();

        assertEquals(0, stackOperations.size());
    }

    @Test
    public void run_With3PatientIdsBatchSize3_ReturnsOneSupplier() {
        String prefix = "01";
        Status90RegistryPatientIdPrefixCleanupSupplier status90RegistryPatientIdPrefixCleanupSupplier =
                new Status90RegistryPatientIdPrefixCleanupSupplier(registryDeletionService, cleanupOperationsFactory, 3, prefix);
        List<String> patientIds = Arrays.asList("1", "2", "3");
        Mockito.when(registryDeletionService.getDocumentEntryPatientIdsWithPrefix(Mockito.anyString()))
                .thenReturn(patientIds);
        Status90RegistryPatientIdBatchCleanupSupplier cleanupSupplier = Mockito.mock(Status90RegistryPatientIdBatchCleanupSupplier.class);
        Mockito.when(cleanupOperationsFactory.createStatus90RegistryPatientIdBatchCleanupSupplier(Mockito.argThat(new ArgumentMatcher<List<String>>() {
            @Override
            public boolean matches(Object argument) {
                List<String> list = (List<String>) argument;
                return list.size() == patientIds.size() && list.containsAll(patientIds);
            }
        }))).thenReturn(cleanupSupplier);

        List<StackOperation> stackOperations = status90RegistryPatientIdPrefixCleanupSupplier.run();

        assertEquals(1, stackOperations.size());
        assertTrue(stackOperations.contains(cleanupSupplier));
    }

    @Test
    public void run_With3PatientIdsBatchSize2_ReturnsTwoSuppliers() {
        String prefix = "01";
        Status90RegistryPatientIdPrefixCleanupSupplier status90RegistryPatientIdPrefixCleanupSupplier =
                new Status90RegistryPatientIdPrefixCleanupSupplier(registryDeletionService, cleanupOperationsFactory, 2, prefix);
        List<String> patientIds = Arrays.asList("1", "2", "3");
        Mockito.when(registryDeletionService.getDocumentEntryPatientIdsWithPrefix(Mockito.anyString()))
                .thenReturn(patientIds);
        Status90RegistryPatientIdBatchCleanupSupplier cleanupSupplier1 = Mockito.mock(Status90RegistryPatientIdBatchCleanupSupplier.class);
        Status90RegistryPatientIdBatchCleanupSupplier cleanupSupplier2 = Mockito.mock(Status90RegistryPatientIdBatchCleanupSupplier.class);
        Mockito.when(cleanupOperationsFactory.createStatus90RegistryPatientIdBatchCleanupSupplier(Mockito.anyList()))
                .thenReturn(cleanupSupplier1).thenReturn(cleanupSupplier2);

        List<StackOperation> stackOperations = status90RegistryPatientIdPrefixCleanupSupplier.run();

        assertEquals(2, stackOperations.size());
        assertTrue(stackOperations.contains(cleanupSupplier1));
        assertTrue(stackOperations.contains(cleanupSupplier2));
        ArgumentCaptor<List> patientIdsCaptor = ArgumentCaptor.forClass(List.class);
        Mockito.verify(cleanupOperationsFactory, Mockito.times(2)).createStatus90RegistryPatientIdBatchCleanupSupplier(patientIdsCaptor.capture());
        List<String> patientIdArguments = (List<String>) patientIdsCaptor.getAllValues().stream().flatMap(list -> list.stream()).collect(Collectors.toList());
        assertEquals(patientIds.size(), patientIdArguments.size());
        assertTrue(patientIdArguments.containsAll(patientIds));
    }

    @Test
    public void run_WhenRetrievingDocumentEntriesFails_ThrowsCleanupException() {
        Mockito.when(registryDeletionService.getDocumentEntryPatientIdsWithPrefix(Mockito.anyString())).thenThrow(RuntimeException.class);
        Status90RegistryPatientIdPrefixCleanupSupplier status90RegistryPatientIdPrefixCleanupSupplier =
                new Status90RegistryPatientIdPrefixCleanupSupplier(registryDeletionService, cleanupOperationsFactory, 2, "01");

        CleanupException e = assertThrows(CleanupException.class, () -> status90RegistryPatientIdPrefixCleanupSupplier.run());

        assertEquals("Failed to retrieve document entry patient ids with prefix 01", e.getMessage());
    }
}