package dk.nsp.xdscleanup.stack.registry.timebased;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.common.model.xds.TypeCode;
import dk.nsp.xdscleanup.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.model.DocumentTypeCleanupSpecification;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.stack.registry.RegistryBatchCleanupOperation;
import dk.nsp.xdscleanup.common.util.CleanupException;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Mockito;

import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import static dk.nsp.xdscleanup.TestUtils.assertThrows;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class TypeAndTimeBasedRegistryCleanupSupplierTest {
    private static final int BATCH_SIZE = 2;
    private static final int LIMIT = 1000;
    private DocumentTypeCleanupSpecification typeSpecification;
    private RegistryDeletionService registryDeletionService;
    private CleanupOperationsFactory cleanupOperationsFactory;
    private TypeAndTimeBasedRegistryCleanupSupplier typeAndTimeBasedRegistryCleanupSupplier;
    private DateTimeProvider dateTimeProvider;

    @Before
    public void setUp() throws Exception {
        typeSpecification = Mockito.mock(DocumentTypeCleanupSpecification.class);
        registryDeletionService = Mockito.mock(RegistryDeletionService.class);
        dateTimeProvider = Mockito.mock(DateTimeProvider.class);
        cleanupOperationsFactory = Mockito.mock(CleanupOperationsFactory.class);
        typeAndTimeBasedRegistryCleanupSupplier = new TypeAndTimeBasedRegistryCleanupSupplier(
                typeSpecification, registryDeletionService, dateTimeProvider, BATCH_SIZE, cleanupOperationsFactory, LIMIT);
    }

    @Test
    public void run_With5EntriesAndBatchSize2_Returns3BatchOperations() {
        // Arrange
        DocumentEntry documentEntry1 = new DocumentEntry(1, "1", "1");
        DocumentEntry documentEntry2 = new DocumentEntry(2, "2", "2");
        DocumentEntry documentEntry3 = new DocumentEntry(3, "3", "3");
        DocumentEntry documentEntry4 = new DocumentEntry(4, "4", "4");
        DocumentEntry documentEntry5 = new DocumentEntry(5, "5", "5");
        RegistryBatchCleanupOperation operation1 = Mockito.mock(RegistryBatchCleanupOperation.class);
        Mockito.when(operation1.withOperationName(Mockito.anyString())).thenReturn(operation1);
        RegistryBatchCleanupOperation operation2 = Mockito.mock(RegistryBatchCleanupOperation.class);
        Mockito.when(operation2.withOperationName(Mockito.anyString())).thenReturn(operation2);
        RegistryBatchCleanupOperation operation3 = Mockito.mock(RegistryBatchCleanupOperation.class);
        Mockito.when(operation3.withOperationName(Mockito.anyString())).thenReturn(operation3);
        Mockito.when(dateTimeProvider.now()).thenReturn(ZonedDateTime.now());
        Mockito.when(typeSpecification.getNumberOfMonthsToKeep()).thenReturn(5);
        Mockito.when(typeSpecification.getTypeCode()).thenReturn(new TypeCode("jens", "1234"));
        Mockito.when(registryDeletionService.getDocumentEntriesForDeletion(Mockito.any(), Mockito.any(), Mockito.anyInt()))
                .thenReturn(Arrays.asList(documentEntry1, documentEntry2, documentEntry3, documentEntry4, documentEntry5));
        Mockito.when(cleanupOperationsFactory.createRegistryBatchCleanupOperation(Mockito.anyList()))
                .thenReturn(operation1).thenReturn(operation2).thenReturn(operation3);

        // Act
        List<StackOperation> stackOperations = typeAndTimeBasedRegistryCleanupSupplier.run();

        // Assert
        assertEquals(3, stackOperations.size());
        assertTrue(stackOperations.contains(operation1));
        assertTrue(stackOperations.contains(operation2));
        assertTrue(stackOperations.contains(operation3));
        ArgumentCaptor<List> documentEntryCaptor = ArgumentCaptor.forClass(List.class);
        Mockito.verify(cleanupOperationsFactory, Mockito.times(3)).createRegistryBatchCleanupOperation(documentEntryCaptor.capture());
        List documentEntries = (List) documentEntryCaptor.getAllValues().stream().flatMap(entries -> entries.stream())
                .collect(Collectors.toList());
        assertEquals(5, documentEntries.size());
        assertTrue(documentEntries.contains(documentEntry1));
        assertTrue(documentEntries.contains(documentEntry2));
        assertTrue(documentEntries.contains(documentEntry3));
        assertTrue(documentEntries.contains(documentEntry4));
        assertTrue(documentEntries.contains(documentEntry5));
    }

    @Test
    public void run_With0EntriesAndBatchSize2_Returns0BatchOperations() {
        // Arrange
        Mockito.when(dateTimeProvider.now()).thenReturn(ZonedDateTime.now());
        Mockito.when(typeSpecification.getNumberOfMonthsToKeep()).thenReturn(5);
        Mockito.when(typeSpecification.getTypeCode()).thenReturn(new TypeCode("jens", "1234"));
        Mockito.when(registryDeletionService.getDocumentEntriesForDeletion(Mockito.any(), Mockito.any(), Mockito.anyInt()))
                .thenReturn(Collections.emptyList());

        // Act
        List<StackOperation> stackOperations = typeAndTimeBasedRegistryCleanupSupplier.run();

        // Assert
        assertEquals(0, stackOperations.size());
        Mockito.verifyZeroInteractions(cleanupOperationsFactory);
    }

    @Test
    public void run_RetrievesCorrectDocumentEntriesForDeletion() {
        // Arrange
        ZonedDateTime now = ZonedDateTime.now();
        int monthsToKeep = 5;
        TypeCode typeCode = new TypeCode("jens", "1234");
        Mockito.when(dateTimeProvider.now()).thenReturn(now);
        Mockito.when(typeSpecification.getNumberOfMonthsToKeep()).thenReturn(monthsToKeep);
        Mockito.when(typeSpecification.getTypeCode()).thenReturn(typeCode);

        // Act
        typeAndTimeBasedRegistryCleanupSupplier.run();

        // Assert
        Mockito.verify(registryDeletionService).getDocumentEntriesForDeletion(typeCode, now.minusMonths(monthsToKeep).toInstant(), LIMIT);
    }

    @Test
    public void run_WhenRetrievingDocumentEntriesFails_ThrowsCleanupException() {
        Mockito.when(dateTimeProvider.now()).thenReturn(ZonedDateTime.now());
        Mockito.when(registryDeletionService.getDocumentEntriesForDeletion(Mockito.any(), Mockito.any(), Mockito.anyInt())).thenThrow(RuntimeException.class);

        CleanupException e = assertThrows(CleanupException.class, () -> typeAndTimeBasedRegistryCleanupSupplier.run());

        assertTrue(e.getMessage(), e.getMessage().contains("Failed to retrieve document entries with latestStartTime"));
    }
}