package dk.nsp.xdscleanup.stack.repository;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.common.util.CleanupException;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Mockito;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import static dk.nsp.xdscleanup.TestUtils.assertThrows;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class RepositoryCleanupSupplierTest {
    private static final int BATCH_SIZE = 2;
    private static final int LIMIT = 1000;
    private RegistryDeletionService registryDeletionService;
    private CleanupOperationsFactory cleanupOperationsFactory;
    private RepositoryCleanupSupplier repositoryCleanupSupplier;

    @Before
    public void setUp() throws Exception {
        registryDeletionService = Mockito.mock(RegistryDeletionService.class);
        cleanupOperationsFactory = Mockito.mock(CleanupOperationsFactory.class);
        repositoryCleanupSupplier = new RepositoryCleanupSupplier(registryDeletionService, cleanupOperationsFactory, BATCH_SIZE, LIMIT);
    }

    @Test
    public void run_With5DocumentEntriesAndBatchSize3_Returns3BatchOperations() {
        DocumentEntry documentEntry1 = new DocumentEntry(1, "1", "1");
        DocumentEntry documentEntry2 = new DocumentEntry(2, "2", "2");
        DocumentEntry documentEntry3 = new DocumentEntry(3, "3", "3");
        DocumentEntry documentEntry4 = new DocumentEntry(4, "4", "4");
        DocumentEntry documentEntry5 = new DocumentEntry(5, "5", "5");
        Mockito.when(registryDeletionService.findDeletedDocumentEntries(LIMIT))
                .thenReturn(Arrays.asList(documentEntry1, documentEntry2, documentEntry3, documentEntry4, documentEntry5));
        RepositoryBatchCleanupOperation operation1 = Mockito.mock(RepositoryBatchCleanupOperation.class);
        RepositoryBatchCleanupOperation operation2 = Mockito.mock(RepositoryBatchCleanupOperation.class);
        RepositoryBatchCleanupOperation operation3 = Mockito.mock(RepositoryBatchCleanupOperation.class);
        Mockito.when(cleanupOperationsFactory.createRepositoryBatchCleanupOperation(Mockito.anyList()))
                .thenReturn(operation1).thenReturn(operation2).thenReturn(operation3);

        List<StackOperation> stackOperations = repositoryCleanupSupplier.run();

        assertEquals(3, stackOperations.size());
        assertTrue(stackOperations.contains(operation1));
        assertTrue(stackOperations.contains(operation2));
        assertTrue(stackOperations.contains(operation3));
        ArgumentCaptor<List> documentEntryCaptor = ArgumentCaptor.forClass(List.class);
        Mockito.verify(cleanupOperationsFactory, Mockito.times(3)).createRepositoryBatchCleanupOperation(documentEntryCaptor.capture());
        List documentEntries = (List) documentEntryCaptor.getAllValues().stream().flatMap(entries -> entries.stream())
                .collect(Collectors.toList());
        assertEquals(5, documentEntries.size());
        assertTrue(documentEntries.contains(documentEntry1));
        assertTrue(documentEntries.contains(documentEntry2));
        assertTrue(documentEntries.contains(documentEntry3));
        assertTrue(documentEntries.contains(documentEntry4));
        assertTrue(documentEntries.contains(documentEntry5));
    }

    @Test
    public void run_With0DocumentEntriesAndBatchSize3_Returns0StackOperations() {
        Mockito.when(registryDeletionService.findDeletedDocumentEntries(LIMIT)).thenReturn(Collections.emptyList());

        List<StackOperation> stackOperations = repositoryCleanupSupplier.run();

        assertEquals(0, stackOperations.size());
    }

    @Test
    public void run_WhenRetrievingDocumentEntriesFails_ThrowsCleanupException() {
        Mockito.doThrow(RuntimeException.class).when(registryDeletionService).findDeletedDocumentEntries(LIMIT);

        CleanupException e = assertThrows(CleanupException.class, () -> repositoryCleanupSupplier.run());

        assertEquals("Failed to retrieve deleted document entries", e.getMessage());
    }
}