package dk.nsp.xdscleanup.nxrg.dao.impl;

import dk.nsp.backgroundjob.health.HealthIndication;
import dk.nsp.backgroundjob.health.HealthIndicator;
import dk.nsp.xdscleanup.common.model.Association;
import dk.nsp.xdscleanup.common.model.DeletionStatus;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.common.model.SubmissionSet;
import dk.nsp.xdscleanup.common.model.xds.AssociationType;
import dk.nsp.xdscleanup.common.model.xds.ReferenceType;
import dk.nsp.xdscleanup.common.model.xds.TypeCode;
import dk.nsp.xdscleanup.common.util.DateTimeUtils;
import dk.nsp.xdscleanup.nxrg.dao.NxrgDao;
import org.apache.commons.lang3.tuple.ImmutablePair;
import org.apache.commons.lang3.tuple.ImmutableTriple;
import org.apache.commons.lang3.tuple.Triple;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.sql.DataSource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.Instant;
import java.util.*;
import java.util.stream.Collectors;

public class NxrgDaoImpl implements NxrgDao, HealthIndicator {
    private final DataSource dataSource;

    private final int deletionMaxAttempts;
    private final int inClauseMaxSize;

    public NxrgDaoImpl(DataSource dataSource, int deletionMaxAttempts, int inClauseMaxSize) {
        this.dataSource = dataSource;
        this.deletionMaxAttempts = deletionMaxAttempts;
        this.inClauseMaxSize = inClauseMaxSize;
    }

    @Override
    public List<DocumentEntry> getDocumentEntriesForCitizens(List<String> citizenCprs) {
        if (citizenCprs.isEmpty()) {
            return Collections.emptyList();
        }

        List<DocumentEntry> documentEntries = new ArrayList<>();
        // The list may be long, so we split it into chunks.
        for(int i = 0; i < citizenCprs.size(); i += inClauseMaxSize) {
            documentEntries.addAll(getDocumentEntriesForCitizensBatch(citizenCprs.subList(i, Math.min(i + inClauseMaxSize, citizenCprs.size()))));
        }
        return documentEntries;
    }

    @Override
    public List<DocumentEntry> getDocumentEntriesForDeletion(TypeCode typeCode, Instant latestStartTime, int limit) {
        String query = "select id, entryuuid, uniqueid from documententries where typecode_codename = :typeCodeCodeName and typecode_schemename = :typeCodeSchemeName and deletetrigger_time < :latestStartTime limit :limit";

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("typeCodeCodeName", typeCode.getCodeName());
        parameters.addValue("typeCodeSchemeName", typeCode.getCodeScheme());
        parameters.addValue("latestStartTime", DateTimeUtils.format(latestStartTime));
        parameters.addValue("limit", limit);

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        return template.query(query, parameters, (rs, rowNum) -> mapDocumentEntryRow(rs));
    }

    @Override
    public List<DocumentEntry> getDeprecatedDocumentsForDeletion(String patientIdPrefix, TypeCode typeCode, int daysBeforeDelete, int limit) {

        //fordi parameterisering ikke fungerer i h2 databasen i interval konstellationen, laves almindelig string replace for variablen daysBeforeDelete
        String daysBeforeDeleteString = Integer.toString(daysBeforeDelete);

        String query = "select de.id, de.entryuuid, de.uniqueid from documententries de " +
                "inner join associations as as_t_repl on as_t_repl.targetuuid_documententry = de.entryuuid " +
                    "and as_t_repl.association_type in ('REPLACE', 'UPDATE_AVAILABILITY_STATUS') " +
                "left join associations as as_s_repl on as_s_repl.sourceuuid_documententry = de.entryuuid " +
                    "and as_s_repl.association_type = 'REPLACE' " +
                "where de.patientid like :patientIdPrefix " +
                    "and as_t_repl.creation_time < (CURRENT_TIMESTAMP - INTERVAL " + daysBeforeDeleteString + " DAY) " +
                    "and de.availabilitystatus = 'Deprecated' " +
                    "and de.typecode_codename = :typeCodeCodeName and de.typecode_schemename = :typeCodeSchemeName " +
                    "and as_s_repl.id is null limit :limit";

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("patientIdPrefix", patientIdPrefix + "%");
        parameters.addValue("typeCodeCodeName", typeCode.getCodeName());
        parameters.addValue("typeCodeSchemeName", typeCode.getCodeScheme());
        parameters.addValue("daysBeforeDelete", daysBeforeDelete);
        parameters.addValue("limit", limit);

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        return template.query(query, parameters, (rs, rowNum) -> mapDocumentEntryRow(rs));
    }

    @Override
    public List<SubmissionSet> getDanglingSubmissionSetsForDeletion(int limit) {
        String query = "select ss.id, ss.entryuuid from submissionsets ss left join associations assoc on assoc.sourceuuid_submissionset=ss.entryuuid where assoc.id is null limit :limit";

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("limit", limit);

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        return template.query(query, parameters, (rs, rowNum) -> mapSubmissionSetsRow(rs));
    }

    @Override
    public List<String> getDocumentEntryPatientIdsWithPrefix(String prefix) {
        String query = "select distinct patientid from documententries" +
                "       where patientid like :prefix";
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("prefix", prefix + "%");

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        return template.queryForList(query, parameters, String.class);
    }

    @Override
    @Transactional(propagation = Propagation.MANDATORY, transactionManager = "tx-nxrg")
    public List<Association> getAssociationsForDocumentEntries(List<String> documentEntryUuids) {
        if(documentEntryUuids == null || documentEntryUuids.isEmpty()) {
            return Arrays.asList();
        }

        List<Association> associations = new ArrayList<>();

        // The list may be long, so we split it into chunks.
        for(int i = 0; i < documentEntryUuids.size(); i += inClauseMaxSize) {
            associations.addAll(getAssociationsForDocumentEntriesBatch(documentEntryUuids.subList(i, Math.min(i + inClauseMaxSize, documentEntryUuids.size()))));
        }

        return associations;
    }

    @Override
    @Transactional(propagation = Propagation.MANDATORY, transactionManager = "tx-nxrg")
    public List<SubmissionSet> determineSubmissionSetsLeftEmpty(List<String> submissionSetEntryUuids, List<String> documentEntryUuidsForDeletion) {
        if(submissionSetEntryUuids == null || submissionSetEntryUuids.isEmpty() || documentEntryUuidsForDeletion == null || documentEntryUuidsForDeletion.isEmpty()) {
            return Arrays.asList();
        }

        List<SubmissionSet> submissionSets = new ArrayList<>();

        // The list may be long, so we split it into chunks.
        for(int i = 0; i < submissionSetEntryUuids.size(); i += inClauseMaxSize) {
            submissionSets.addAll(determineSubmissionSetsLeftEmptyBatch(submissionSetEntryUuids.subList(i, Math.min(i + inClauseMaxSize, submissionSetEntryUuids.size())), documentEntryUuidsForDeletion));
        }

        return submissionSets;
    }

    @Override
    @Transactional(propagation = Propagation.MANDATORY, transactionManager = "tx-nxrg")
    public void deleteDocumentEntries(List<Long> documentEntryIds) {
        if(documentEntryIds == null || documentEntryIds.isEmpty()) {
            return;
        }

        // The list may be long, so we split it into chunks.
        for(int i = 0; i < documentEntryIds.size(); i += inClauseMaxSize) {
            deleteDocumentEntryBatch(documentEntryIds.subList(i, Math.min(i + inClauseMaxSize, documentEntryIds.size())));
        }
    }

    @Override
    @Transactional(propagation = Propagation.MANDATORY, transactionManager = "tx-nxrg")
    public void deleteAssociations(List<Long> associationIds) {
        if(associationIds == null || associationIds.isEmpty()) {
            return;
        }

        // The list may be long, so we split it into chunks.
        for(int i = 0; i < associationIds.size(); i += inClauseMaxSize) {
            deleteAssociationBatch(associationIds.subList(i, Math.min(i + inClauseMaxSize, associationIds.size())));
        }
    }

    @Override
    @Transactional(propagation = Propagation.MANDATORY, transactionManager = "tx-nxrg")
    public void deleteSubmissionSets(List<Long> submissionSetIds) {
        if(submissionSetIds == null || submissionSetIds.isEmpty()) {
            return;
        }

        // The list may be long, so we split it into chunks.
        for(int i = 0; i < submissionSetIds.size(); i += inClauseMaxSize) {
            deleteSubmissionSetBatch(submissionSetIds.subList(i, Math.min(i + inClauseMaxSize, submissionSetIds.size())));
        }
    }

    @Override
    public List<DocumentEntry> getDocumentEntriesForRepositoryDeletion(int limit) {
        String query = "select id, entryuuid, uniqueid from deleted_documententries limit :limit";

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("limit", limit);

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        return template.query(query, parameters, (rs, rowNum) -> mapDocumentEntryRow(rs));
    }

    @Override
    @Transactional(propagation = Propagation.MANDATORY, transactionManager = "tx-nxrg")
    public void markSuccessfulRegistryDeletion(List<DocumentEntry> deletedDocumentEntries) {
        if(deletedDocumentEntries == null || deletedDocumentEntries.isEmpty()) {
            return;
        }

        String command = "insert into deleted_documententries(entryuuid, uniqueid, deletion_status, deletion_attempts) values (:entryUuid, :uniqueId, :deletionStatus, :deletionAttempts)";

        MapSqlParameterSource[] parameterSources = deletedDocumentEntries
                .stream()
                .map(dde -> toParameterSource(dde))
                .collect(Collectors.toList())
                .toArray(new MapSqlParameterSource[deletedDocumentEntries.size()]);

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        template.batchUpdate(command, parameterSources);
    }

    @Override
    @Transactional(propagation = Propagation.MANDATORY, transactionManager = "tx-nxrg")
    public void markSuccessfulRepositoryDeletion(List<String> documentEntryUuids) {
        if(documentEntryUuids == null || documentEntryUuids.isEmpty()) {
            return;
        }

        // The list may be long, so we split it into chunks.
        for(int i = 0; i < documentEntryUuids.size(); i += inClauseMaxSize) {
            deleteFromDeletedDocumentEntries(documentEntryUuids.subList(i, Math.min(i + inClauseMaxSize, documentEntryUuids.size())));
        }
    }

    @Override
    @Transactional(propagation = Propagation.MANDATORY, transactionManager = "tx-nxrg")
    public void markFailedRepositoryDeletion(List<String> documentEntryUuids) {
        if(documentEntryUuids == null || documentEntryUuids.isEmpty()) {
            return;
        }

        // The list may be long, so we split it into chunks.
        for(int i = 0; i < documentEntryUuids.size(); i += inClauseMaxSize) {
            markFailedRepositoryDeletionBatch(documentEntryUuids.subList(i, Math.min(i + inClauseMaxSize, documentEntryUuids.size())));
        }
    }

    @Override
    public HealthIndication getHealthInfo() {
		String registryDb = "registry.db";

        String failedDeletionsQuery = String.format("select (select count(*) from deleted_documententries where deletion_status = '%s' and deletion_attempts >= :deletionMaxAttempts) as failed_repository_deletions, (select count(*) from deleted_documententries) as enqueued_repository_deletions", DeletionStatus.DELETION_FROM_REPOSITORY_FAILED);


        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("deletionMaxAttempts",  deletionMaxAttempts);

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);

        int failedRepositoryDeletions = 0;
        int enqueuedRepositoryDeletions = 0;
		try {
            ImmutablePair<Integer, Integer> counts = template.query(failedDeletionsQuery, parameters, (rs, rowNum) -> new ImmutablePair<>(rs.getInt("failed_repository_deletions"), rs.getInt("enqueued_repository_deletions"))).get(0);
            failedRepositoryDeletions = counts.getLeft();
            enqueuedRepositoryDeletions = counts.getRight();
		}
        catch (DataAccessException e) {
			return new HealthIndication(500, "\""+registryDb+"\": \"ERROR: Database connection dead.\"");
		}

        if(failedRepositoryDeletions > 0) {
            return new HealthIndication(500,
                    String.format("\"%s\": \"ERROR: %d entries could not be deleted from repository (at least %d attempts).\"", registryDb, failedRepositoryDeletions, deletionMaxAttempts));
        }

		return new HealthIndication(200, "\""+registryDb+"\": \"OK: " + enqueuedRepositoryDeletions + " entries currently enqueued for repository deletion.\"");
	}

    private List<DocumentEntry> getDocumentEntriesForCitizensBatch(List<String> citizenCprs) {
        String query = "select id, entryuuid, uniqueid from documententries where patientid in (:cprs)";

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("cprs", citizenCprs);

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        return template.query(query, parameters, (rs, rowNum) -> mapDocumentEntryRow(rs));
    }

    private List<Association> getAssociationsForDocumentEntriesBatch(List<String> documentEntryUuids) {
        String query = "select id, entryuuid, association_type, sourceuuid_documentEntry, sourceuuid_submissionset, targetuuid_documentEntry, targetuuid_submissionset from associations where sourceuuid_documententry in (:documentEntryUuids) or targetuuid_documententry in (:documentEntryUuids)";

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("documentEntryUuids", documentEntryUuids);

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        return template.query(query, parameters, (rs, rowNum) -> mapAssociationRow(rs));
    }

    private List<SubmissionSet> determineSubmissionSetsLeftEmptyBatch(List<String> submissionSetEntryUuids, List<String> documentEntryUuidsForDeletion) {
        List<SubmissionSet> submissionSets = new ArrayList<>();

        // Determine which DocumentEntries are contained in each SubmissionSet.
        Map<String, Triple<Long, String, List<String>>> documentEntryUuidsBySubmissionSetUuid = getDocumentEntryUuidsBySubmissionSetEntryUuid(submissionSetEntryUuids);

        // Find those SubmissionSets that will be left empty by the deletion.
        for(String submissionSetEntryUuid : documentEntryUuidsBySubmissionSetUuid.keySet()) {
            Triple<Long, String, List<String>> submissionSetData = documentEntryUuidsBySubmissionSetUuid.get(submissionSetEntryUuid);
            List<String> documentEntryUuids = documentEntryUuidsBySubmissionSetUuid.get(submissionSetEntryUuid).getRight();

            // Check whether every DocumentEntry of the SubmissionSet is to be deleted.
            if(documentEntryUuidsForDeletion.containsAll(documentEntryUuids)) {
                submissionSets.add(new SubmissionSet(submissionSetData.getLeft(), submissionSetData.getMiddle()));
            }
        }

        return submissionSets;
    }

    private Map<String, Triple<Long, String, List<String>>> getDocumentEntryUuidsBySubmissionSetEntryUuid(List<String> submissionSetEntryUuids) {
        String query = "select ss.id, ss.entryuuid, a.targetuuid_documententry from submissionsets ss inner join associations a on ss.entryuuid = a.sourceuuid_submissionset where ss.entryuuid in (:submissionSetEntryUuids) and a.targetuuid_documententry is not null";

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("submissionSetEntryUuids", submissionSetEntryUuids);

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        List<Map<String, Object>> rows = template.queryForList(query, parameters);

        Map<String, Triple<Long, String, List<String>>> result = new HashMap<>();

        for(Map<String, Object> row : rows) {

            // TODO SDS-7283: Dette skal erstattes af "submissionSetId = (Long) row.get("id");" når felterne er rettet til bigint alle steder.
            long submissionSetId;
            if (Long.class.isInstance(row.get("id"))) {
                submissionSetId = (Long) row.get("id");
            } else {
                submissionSetId = (Integer) row.get("id");
            }

            String submissionSetEntryUuid = (String) row.get("entryuuid");
            String documentEntryUuid = (String) row.get("targetuuid_documententry");

            if(!result.containsKey(submissionSetEntryUuid)) {
                result.put(submissionSetEntryUuid, new ImmutableTriple<>(submissionSetId, submissionSetEntryUuid, new ArrayList<>()));
            }
            result.get(submissionSetEntryUuid).getRight().add(documentEntryUuid);
        }

        return result;
    }

    private void deleteDocumentEntryBatch(List<Long> documentEntryIds) {
        // The command also deletes from the tables 'documententries', 'documententry_author', 'documententry_eventcode',
        // 'documententry_confidentialitycode' and 'documententry_referenceid'.
        // This is due to 'on delete cascade'-constraints (see the table definitions).
        String command = "delete dc from documententrycontent dc where dc.id in (select de.documententrycontentid from documententries de where de.id in (:documentEntryIds))";

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("documentEntryIds", documentEntryIds);

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        template.update(command, parameters);
    }

    private void deleteAssociationBatch(List<Long> associationIds) {
        // The command also deletes from the table 'associations'.
        // This is due to 'on delete cascade'-constraints (see the table definitions).
        String command = "delete ac from associationcontent ac where ac.id in (select a.associationcontentid from associations a where a.id in (:associationIds))";


        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("associationIds", associationIds);

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        template.update(command, parameters);
    }

    private void deleteSubmissionSetBatch(List<Long> submissionSetIds) {
        // The command also deletes from the table 'submissionsets'.
        // This is due to 'on delete cascade'-constraints (see the table definitions).

        // The delete has been split in two: first select the data and then delete it. This is because it performs better in a Master/Slave/Backup nodes setup.

        List<Long> submissionSetContentIds =  getSubmissionSetContentIdsForDeletion(submissionSetIds);
        if (submissionSetContentIds.size() > 0) {
            String command = "delete sc from submissionsetcontent sc where sc.id in (:submissionSetContentIds)";

            MapSqlParameterSource parameters = new MapSqlParameterSource();
            parameters.addValue("submissionSetContentIds", submissionSetContentIds);

            NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
            template.update(command, parameters);
        }
    }

    private List<Long> getSubmissionSetContentIdsForDeletion(List<Long> submissionSetIds) {
        String query = "select s.submissionsetcontentid  from submissionsets s where s.id in (:submissionSetIds)";

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("submissionSetIds", submissionSetIds);

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        return template.queryForList(query, parameters, Long.class);
    }

    private void deleteFromDeletedDocumentEntries(List<String> documentEntryUuids) {
        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);

        String command = "delete from deleted_documententries where entryUuid in (:documentEntryUuids)";

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("documentEntryUuids", documentEntryUuids);

        template.update(command, parameters);
    }

    private void markFailedRepositoryDeletionBatch(List<String> documentEntryUuids) {
        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        checkExistenceOfEntries(documentEntryUuids, template);

        String command = "update deleted_documententries set deletion_status = :deletionStatus, deletion_attempts = deletion_attempts + 1 where entryUuid in (:documentEntryUuids)";

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("deletionStatus", DeletionStatus.DELETION_FROM_REPOSITORY_FAILED.toString());
        parameters.addValue("documentEntryUuids", documentEntryUuids);

        template.update(command, parameters);
    }

    private DocumentEntry mapDocumentEntryRow(ResultSet rs) throws SQLException {
        return new DocumentEntry(rs.getLong("id"), rs.getString("entryuuid"), rs.getString("uniqueid"));
    }

    private SubmissionSet mapSubmissionSetsRow(ResultSet rs) throws SQLException {
        return new SubmissionSet(rs.getLong("id"), rs.getString("entryuuid"));
    }

    private Association mapAssociationRow(ResultSet rs) throws SQLException {
        long entryId = rs.getLong("id");
        String entryUuid = rs.getString("entryuuid");
        AssociationType associationType = Enum.valueOf(AssociationType.class, rs.getString("association_type"));

        String sourceUuid = null;
        ReferenceType sourceType = null;
        String sourceUuidDocumentEntry = rs.getString("sourceuuid_documentEntry");
        if(sourceUuidDocumentEntry != null && !sourceUuidDocumentEntry.isEmpty()) {
            sourceUuid = sourceUuidDocumentEntry;
            sourceType = ReferenceType.DOCUMENT_ENTRY;
        }
        String sourceUuidSubmissionSet = rs.getString("sourceuuid_submissionset");
        if(sourceUuidSubmissionSet != null && !sourceUuidSubmissionSet.isEmpty()) {
            sourceUuid = sourceUuidSubmissionSet;
            sourceType = ReferenceType.SUBMISSION_SET;
        }

        String targetUuid = null;
        ReferenceType targetType  = null;
        String targetUuidDocumentEntry = rs.getString("targetuuid_documentEntry");
        if(targetUuidDocumentEntry != null && !targetUuidDocumentEntry.isEmpty()) {
            targetUuid = targetUuidDocumentEntry;
            targetType = ReferenceType.DOCUMENT_ENTRY;
        }
        String targetUuidSubmissionSet = rs.getString("targetuuid_submissionset");
        if(targetUuidSubmissionSet != null && !targetUuidSubmissionSet.isEmpty()) {
            targetUuid = targetUuidSubmissionSet;
            targetType = ReferenceType.SUBMISSION_SET;
        }

        return new Association(entryId, entryUuid, associationType, sourceUuid, sourceType, targetUuid, targetType);
    }

    private MapSqlParameterSource toParameterSource(DocumentEntry dde) {
        MapSqlParameterSource parameters = new MapSqlParameterSource();

        parameters.addValue("entryUuid", dde.getEntryUuid());
        parameters.addValue("uniqueId", dde.getUniqueId());
        parameters.addValue("deletionStatus", DeletionStatus.DELETED_FROM_REGISTRY.toString());
        parameters.addValue("deletionAttempts", 0);

        return parameters;
    }

    private void checkExistenceOfEntries(List<String> documentEntryUuids, NamedParameterJdbcTemplate template) {
        List<String> existingDocumentEntryUuids = template.query("select entryuuid from deleted_documententries where entryuuid in (:documentEntryUuids)",
                new MapSqlParameterSource("documentEntryUuids", documentEntryUuids),
                (rs, rowNum) -> rs.getString("entryuuid"));

        if(!existingDocumentEntryUuids.containsAll(documentEntryUuids)) {
            List<String> unknownEntryUuids = documentEntryUuids.stream().filter(id -> existingDocumentEntryUuids.contains(id)).collect(Collectors.toList());
            throw new IllegalStateException(String.format("Trying to mark deletion of unknown documentEntryUuids: [%s]", String.join(", ", unknownEntryUuids)));
        }
    }
}
