package dk.nsp.xdscleanup.nxrg.service;

import dk.nsp.xdscleanup.common.model.Association;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.common.model.RegistryObject;
import dk.nsp.xdscleanup.common.model.SubmissionSet;
import dk.nsp.xdscleanup.common.model.xds.AssociationType;
import dk.nsp.xdscleanup.common.model.xds.ReferenceType;
import dk.nsp.xdscleanup.common.model.xds.TypeCode;
import dk.nsp.xdscleanup.nxrg.dao.NxrgDao;
import org.apache.log4j.Logger;
import org.springframework.transaction.annotation.Transactional;

import java.time.Instant;
import java.util.List;
import java.util.stream.Collectors;

public class RegistryDeletionService {

    private static final Logger log = Logger.getLogger(RegistryDeletionService.class);
    private final NxrgDao nxrgDao;

	public RegistryDeletionService(NxrgDao nxrgDao) {
		this.nxrgDao = nxrgDao;
    }

    public List<DocumentEntry> getDocumentEntriesForCitizens(List<String> citizenCprs) {
        return nxrgDao.getDocumentEntriesForCitizens(citizenCprs);
    }

    public List<DocumentEntry> getDocumentEntriesForDeletion(TypeCode typeCode, Instant latestStartTime, int limit) {
        return nxrgDao.getDocumentEntriesForDeletion(typeCode, latestStartTime, limit);
    }

    public List<DocumentEntry> getDeprecatedDocumentsForDeletion(String patientIdPrefix, TypeCode typeCode, int daysBeforeDelete, int limit) {
        return nxrgDao.getDeprecatedDocumentsForDeletion(patientIdPrefix, typeCode, daysBeforeDelete, limit);
    }

    public List<SubmissionSet> getDanglingSubmissionSetsForDeletion(int limit) {
        return nxrgDao.getDanglingSubmissionSetsForDeletion(limit);
    }

    public List<String> getDocumentEntryPatientIdsWithPrefix(String prefix) {
        return nxrgDao.getDocumentEntryPatientIdsWithPrefix(prefix);
    }

    @Transactional(transactionManager = "tx-nxrg")
    public void deleteDocumentEntries(List<DocumentEntry> documentEntries) {
        documentEntriesDelete(documentEntries);
    }

    @Transactional(transactionManager = "tx-nxrg")
    public void deleteSubmissionSets(List<SubmissionSet> submissionSets) {
        List<Long> submissionSetIds = submissionSets.stream().map(ss -> ss.getEntryId()).collect(Collectors.toList());
        nxrgDao.deleteSubmissionSets(submissionSetIds);
    }

    public List<DocumentEntry> findDeletedDocumentEntries(int limit) {
        return nxrgDao.getDocumentEntriesForRepositoryDeletion(limit);
    }

    @Transactional(transactionManager = "tx-nxrg")
    public void markDeletionComplete(List<DocumentEntry> deletedDocumentEntries) {
        if(deletedDocumentEntries == null || deletedDocumentEntries.isEmpty()) {
            return;
        }

        List<String> documentEntryUuids = extractEntryUuids(deletedDocumentEntries);
        nxrgDao.markSuccessfulRepositoryDeletion(documentEntryUuids);
    }

    @Transactional(transactionManager = "tx-nxrg")
    public void markDeletionAttemptFailed(List<DocumentEntry> deletedDocumentEntries) {
        if(deletedDocumentEntries == null || deletedDocumentEntries.isEmpty()) {
            return;
        }

        List<String> documentEntryUuids = extractEntryUuids(deletedDocumentEntries);
        nxrgDao.markFailedRepositoryDeletion(documentEntryUuids);
    }

    private List<Association> getAssociationsForDeletion(List<DocumentEntry> documentEntriesForDeletion) {
        List<String> documentEntryUuids = extractEntryUuids(documentEntriesForDeletion);
        return nxrgDao.getAssociationsForDocumentEntries(documentEntryUuids);
    }

    private List<SubmissionSet> getSubmissionSetsForDeletion(List<Association> associationsForDeletion, List<DocumentEntry> documentEntriesForDeletion) {
        List<String> submissionSetUuids = extractSubmissionSetUuids(associationsForDeletion);
        List<String> documentEntryUuids = extractEntryUuids(documentEntriesForDeletion);
        return nxrgDao.determineSubmissionSetsLeftEmpty(submissionSetUuids, documentEntryUuids);
    }

    private List<String> extractEntryUuids(List<? extends RegistryObject> registryObjects) {
        return registryObjects
                .stream()
                .map(de -> de.getEntryUuid())
                .collect(Collectors.toList());
    }

    private List<String> extractSubmissionSetUuids(List<Association> associationsForDeletion) {
        return associationsForDeletion
                .stream()
                .filter(a -> a.getAssociationType() == AssociationType.HAS_MEMBER && a.getSourceType() == ReferenceType.SUBMISSION_SET)
                .map(a -> a.getSourceUuid())
                .collect(Collectors.toList());
    }

    private void deleteRegistryObjects(List<DocumentEntry> documentEntriesForDeletion, List<Association> associationsForDeletion, List<SubmissionSet> submissionSetsForDeletion) {
        // The order matters here, since associations refer to other objects.
        nxrgDao.deleteAssociations(associationsForDeletion.stream().map(a -> a.getEntryId()).collect(Collectors.toList()));
        nxrgDao.deleteDocumentEntries(documentEntriesForDeletion.stream().map(de -> de.getEntryId()).collect(Collectors.toList()));
        nxrgDao.deleteSubmissionSets(submissionSetsForDeletion.stream().map(ss -> ss.getEntryId()).collect(Collectors.toList()));
    }

    private void markDocumentEntriesDeleted(List<DocumentEntry> documentEntriesForDeletion) {
        nxrgDao.markSuccessfulRegistryDeletion(documentEntriesForDeletion);
    }

    private void documentEntriesDelete(List<DocumentEntry> documentEntriesForDeletion) {
        // Retrieve associations that refer to those DocumentEntries
        List<Association> associationsForDeletion = getAssociationsForDeletion(documentEntriesForDeletion);

        // (maybe) Determine any other DocumentEntries referred to by the associations - do they need to be deleted as well?

        // Determine SubmissionSets to delete
        List<SubmissionSet> submissionSetsForDeletion = getSubmissionSetsForDeletion(associationsForDeletion, documentEntriesForDeletion);

        // Delete those objects determined for deletion.
        deleteRegistryObjects(documentEntriesForDeletion, associationsForDeletion, submissionSetsForDeletion);

        // Mark the DocumentEntries as deleted in the status-table.
        markDocumentEntriesDeleted(documentEntriesForDeletion);
    }


}
