package dk.nsp.xdscleanup.nxrg.dao;

import dk.nsp.backgroundjob.health.HealthIndication;
import dk.nsp.xdscleanup.common.model.Association;
import dk.nsp.xdscleanup.common.model.DeletionStatus;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.common.model.SubmissionSet;
import dk.nsp.xdscleanup.common.model.xds.TypeCode;
import dk.nsp.xdscleanup.nxrg.dao.impl.NxrgDaoImpl;
import dk.nsp.xdscleanup.nxrg.setup.EmbeddedDatabaseSetup;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import javax.sql.DataSource;
import java.time.Duration;
import java.time.Instant;
import java.util.*;
import java.util.stream.Collectors;

import static dk.nsp.xdscleanup.nxrg.dao.TestConstants.*;
import static org.junit.Assert.*;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = {EmbeddedDatabaseSetup.class})
public class NxrgDaoTest {
	private NxrgDaoImpl subject;

	private static final int DELETION_MAX_ATTEMPTS = 5;
	private static final int IN_CLAUSE_MAX_SIZE = 1;

	private static final int LIMIT = 1000;

	@Autowired
	private DataSource dataSource;

	@Before
	public void setup() {
		new NxrgTestDataCreator(dataSource).initializeTestDatabase();

		subject = new NxrgDaoImpl(dataSource, DELETION_MAX_ATTEMPTS, IN_CLAUSE_MAX_SIZE);
	}

	@Test
	public void getDocumentEntriesForCitizens_WithEmptyCprList_ReturnsEmptyList() {
		List<DocumentEntry> documentEntries = subject.getDocumentEntriesForCitizens(Collections.emptyList());

		assertTrue(documentEntries.isEmpty());
	}

	@Test
	public void getDocumentEntriesForCitizens_WithCprsWithoutDocuments_ReturnsEmptyList() {
		List<DocumentEntry> documentEntries = subject.getDocumentEntriesForCitizens(Arrays.asList("0000000000", "1111111111"));

		assertTrue(documentEntries.isEmpty());
	}

	@Test
	public void getDocumentEntriesForCitizens_WithCprWithDocument_ReturnsDocumentEntry() {
		List<String> documentEntryUuids = subject.getDocumentEntriesForCitizens(Collections.singletonList(CPR_2)).stream()
				.map(DocumentEntry::getEntryUuid).collect(Collectors.toList());

		assertEquals(Arrays.asList(DOCUMENT_ENTRY_UUID_4), documentEntryUuids);
	}

	@Test
	public void getDocumentEntriesForCitizens_With2OutOf3CprsHavingDocuments_ReturnsDocumentEntries() {
		List<String> documentEntryUuids = subject.getDocumentEntriesForCitizens(Arrays.asList(CPR_2, CPR_3, "0000000000")).stream()
				.map(DocumentEntry::getEntryUuid).collect(Collectors.toList());

		assertEquals(2, documentEntryUuids.size());
		assertTrue(documentEntryUuids.contains(DOCUMENT_ENTRY_UUID_4));
		assertTrue(documentEntryUuids.contains(DOCUMENT_ENTRY_UUID_5));
	}

	@Test
	public void getDocumentEntriesForDeletion_canFilterOnTypeCode_positive() {
		// Arrange
		TypeCode typeCode = TYPE_CODE_0;
		Instant latestStartTime = SERVICE_START_TIME_0.plus(Duration.ofMinutes(5));

		// Act
		List<DocumentEntry> result = subject.getDocumentEntriesForDeletion(typeCode,  latestStartTime, LIMIT);

		// Assert
		assertEquals(3, result.size());
		assertExpectedDocumentEntryExists(result, DOCUMENT_ENTRY_UUID_0);
		assertExpectedDocumentEntryExists(result, DOCUMENT_ENTRY_UUID_2);
		assertExpectedDocumentEntryExists(result, DOCUMENT_ENTRY_UUID_3);
	}

	@Test
	public void getDocumentEntriesForDeletion_canFilterOnTypeCode_negative() {
		// Arrange
		TypeCode typeCode = new TypeCode("Bogus", "Code");
		Instant latestStartTime = SERVICE_START_TIME_0.plus(Duration.ofMinutes(5));

		// Act
		List<DocumentEntry> result = subject.getDocumentEntriesForDeletion(typeCode,  latestStartTime, LIMIT);

		// Assert
		assertEquals(0, result.size());
	}

	@Test
	public void getDocumentEntriesForDeletion_canFilterOnServiceStartTime_positive() {
		// Arrange
		TypeCode typeCode = TYPE_CODE_1;
		Instant latestStartTime = SERVICE_START_TIME_1.plus(Duration.ofMinutes(5));

		// Act
		List<DocumentEntry> result = subject.getDocumentEntriesForDeletion(typeCode,  latestStartTime, LIMIT);

		// Assert
		assertEquals(1, result.size());
		assertExpectedDocumentEntryExists(result, DOCUMENT_ENTRY_UUID_1);
	}

	@Test
	public void getDocumentEntriesForDeletion_canFilterOnServiceStartTime_negative() {
		// Arrange
		TypeCode typeCode = TYPE_CODE_1;
		Instant latestStartTime = SERVICE_START_TIME_1.minus(Duration.ofMinutes(5));

		// Act
		List<DocumentEntry> result = subject.getDocumentEntriesForDeletion(typeCode,  latestStartTime, LIMIT);

		// Assert
		assertEquals(0, result.size());
	}

	@Test
	public void getDocumentEntriesForDeletion_canFilterOnCreationTime_positive() {
		// Arrange
		TypeCode typeCode = TYPE_CODE_0;
		Instant latestStartTime = CREATION_TIME_0.plus(Duration.ofMinutes(5));

		// Act
		List<DocumentEntry> result = subject.getDocumentEntriesForDeletion(typeCode,  latestStartTime, LIMIT);

		// Assert
		assertEquals(1, result.size());
		assertExpectedDocumentEntryExists(result, DOCUMENT_ENTRY_UUID_3);
	}

	@Test
	public void getDocumentEntriesForDeletion_canFilterOnCreationTime_negative() {
		// Arrange
		TypeCode typeCode = TYPE_CODE_0;
		Instant latestStartTime = CREATION_TIME_0.minus(Duration.ofMinutes(5));

		// Act
		List<DocumentEntry> result = subject.getDocumentEntriesForDeletion(typeCode,  latestStartTime, LIMIT);

		// Assert
		assertEquals(0, result.size());
	}

	@Test
	public void getAssociationsForDocumentEntries_null_givesEmptyList() {
		// Arrange

		// Act
		List<Association> result = subject.getAssociationsForDocumentEntries(null);

		// Assert
		assertNotNull(result);
		assertTrue(result.isEmpty());
	}

	@Test
	public void getAssociationsForDocumentEntries_documentEntryIsSource() {
		// Arrange
		List<String> documentEntryUuids = Arrays.asList(DOCUMENT_ENTRY_UUID_0);

		// Act
		List<Association> result = subject.getAssociationsForDocumentEntries(documentEntryUuids);

		// Assert
		assertEquals(2, result.size());
		assertExpectedAssociationExists(result, ASSOCIATION_ENTRY_UUID_0, SUBMISSION_SET_ENTRY_UUID_0, DOCUMENT_ENTRY_UUID_0);
		assertExpectedAssociationExists(result, ASSOCIATION_ENTRY_UUID_4, DOCUMENT_ENTRY_UUID_0, DOCUMENT_ENTRY_UUID_1);
	}

	@Test
	public void getAssociationsForDocumentEntries_documentEntryIsTarget() {
		// Arrange
		List<String> documentEntryUuids = Arrays.asList(DOCUMENT_ENTRY_UUID_2);

		// Act
		List<Association> result = subject.getAssociationsForDocumentEntries(documentEntryUuids);

		// Assert
		assertEquals(1, result.size());
		assertExpectedAssociationExists(result, ASSOCIATION_ENTRY_UUID_2, SUBMISSION_SET_ENTRY_UUID_2, DOCUMENT_ENTRY_UUID_2);
	}

	@Test
	public void getAssociationsForDocumentEntries_ignoresOtherUuids() {
		// Arrange
		List<String> documentEntryUuids = Arrays.asList(SUBMISSION_SET_ENTRY_UUID_0);

		// Act
		List<Association> result = subject.getAssociationsForDocumentEntries(documentEntryUuids);

		// Assert
		assertEquals(0, result.size());
	}

	@Test
	public void getAssociationsForDocumentEntries_multipleDocumentEntries() {
		// Arrange
		List<String> documentEntryUuids = Arrays.asList(DOCUMENT_ENTRY_UUID_0, DOCUMENT_ENTRY_UUID_2);

		// Act
		List<Association> result = subject.getAssociationsForDocumentEntries(documentEntryUuids);

		// Assert
		assertEquals(3, result.size());
		assertExpectedAssociationExists(result, ASSOCIATION_ENTRY_UUID_0, SUBMISSION_SET_ENTRY_UUID_0, DOCUMENT_ENTRY_UUID_0);
		assertExpectedAssociationExists(result, ASSOCIATION_ENTRY_UUID_2, SUBMISSION_SET_ENTRY_UUID_2, DOCUMENT_ENTRY_UUID_2);
		assertExpectedAssociationExists(result, ASSOCIATION_ENTRY_UUID_4, DOCUMENT_ENTRY_UUID_0, DOCUMENT_ENTRY_UUID_1);
	}

	@Test
	public void determineSubmissionSetsLeftEmpty_noSubmissionSets_returnsEmptyList() {
		// Arrange
		List<String> submissionSetEntryUuids = Arrays.asList();
		List<String> documentEntryUuids = Arrays.asList();

		// Act
		List<SubmissionSet> result = subject.determineSubmissionSetsLeftEmpty(submissionSetEntryUuids, documentEntryUuids);

		// Assert
		assertEquals(0, result.size());
	}

	@Test
	public void determineSubmissionSetsLeftEmpty_noDocumentEntries_returnsEmptyList() {
		// Arrange
		List<String> submissionSetEntryUuids = Arrays.asList(SUBMISSION_SET_ENTRY_UUID_0);
		List<String> documentEntryUuids = Arrays.asList();

		// Act
		List<SubmissionSet> result = subject.determineSubmissionSetsLeftEmpty(submissionSetEntryUuids, documentEntryUuids);

		// Assert
		assertEquals(0, result.size());
	}

	@Test
	public void determineSubmissionSetsLeftEmpty_submissionSetLeftEmpty_included() {
		// Arrange
		List<String> submissionSetEntryUuids = Arrays.asList(SUBMISSION_SET_ENTRY_UUID_0);
		List<String> documentEntryUuids = Arrays.asList(DOCUMENT_ENTRY_UUID_0);

		// Act
		List<SubmissionSet> result = subject.determineSubmissionSetsLeftEmpty(submissionSetEntryUuids, documentEntryUuids);

		// Assert
		assertEquals(1, result.size());
		assertExpectedSubmissionSetExists(result, SUBMISSION_SET_ENTRY_UUID_0);
	}

	@Test
	public void determineSubmissionSetsLeftEmpty_submissionSetRemainsNonEmpty_omitted() {
		// Arrange
		List<String> submissionSetEntryUuids = Arrays.asList(SUBMISSION_SET_ENTRY_UUID_0);
		List<String> documentEntryUuids = Arrays.asList(DOCUMENT_ENTRY_UUID_1);

		// Act
		List<SubmissionSet> result = subject.determineSubmissionSetsLeftEmpty(submissionSetEntryUuids, documentEntryUuids);

		// Assert
		assertEquals(0, result.size());
	}

	@Test
	public void determineSubmissionSetsLeftEmpty_submissionSetWithMultipleEntries_leftEmpty() {
		// Arrange
		List<String> submissionSetEntryUuids = Arrays.asList(SUBMISSION_SET_ENTRY_UUID_2);
		List<String> documentEntryUuids = Arrays.asList(DOCUMENT_ENTRY_UUID_2, DOCUMENT_ENTRY_UUID_3);

		// Act
		List<SubmissionSet> result = subject.determineSubmissionSetsLeftEmpty(submissionSetEntryUuids, documentEntryUuids);

		// Assert
		assertEquals(1, result.size());
		assertExpectedSubmissionSetExists(result, SUBMISSION_SET_ENTRY_UUID_2);
	}

	@Test
	public void determineSubmissionSetsLeftEmpty_submissionSetWithMultipleEntries_remainsNonEmpty() {
		// Arrange
		List<String> submissionSetEntryUuids = Arrays.asList(SUBMISSION_SET_ENTRY_UUID_2);
		List<String> documentEntryUuids = Arrays.asList(DOCUMENT_ENTRY_UUID_2);

		// Act
		List<SubmissionSet> result = subject.determineSubmissionSetsLeftEmpty(submissionSetEntryUuids, documentEntryUuids);

		// Assert
		assertEquals(0, result.size());
	}

	@Test
	public void determineSubmissionSetsLeftEmpty_multipleSubmissionSets() {
		// Arrange
		List<String> submissionSetEntryUuids = Arrays.asList(SUBMISSION_SET_ENTRY_UUID_0, SUBMISSION_SET_ENTRY_UUID_2);
		List<String> documentEntryUuids = Arrays.asList(DOCUMENT_ENTRY_UUID_0, DOCUMENT_ENTRY_UUID_1, DOCUMENT_ENTRY_UUID_2, DOCUMENT_ENTRY_UUID_3);

		// Act
		List<SubmissionSet> result = subject.determineSubmissionSetsLeftEmpty(submissionSetEntryUuids, documentEntryUuids);

		// Assert
		assertEquals(2, result.size());
		assertExpectedSubmissionSetExists(result, SUBMISSION_SET_ENTRY_UUID_0);
		assertExpectedSubmissionSetExists(result, SUBMISSION_SET_ENTRY_UUID_2);
	}

	@Test
	public void deleteDocumentEntries_emptyList_noop() {
		// Arrange
		List<Long> documentEntryIds = Arrays.asList();

		// Act
		subject.deleteDocumentEntries(documentEntryIds);

		// Assert
		List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from documententries where entryuuid = '%s'", DOCUMENT_ENTRY_UUID_0));
		assertEquals(1, entries.size());
	}

	@Test
	public void deleteDocumentEntries_nonEmptyList_deletesEntry() {
		// Arrange
		List<Long> documentEntryIds = new JdbcTemplate(dataSource).queryForList(String.format("select id from documententries where entryuuid = '%s'", TestConstants.DOCUMENT_ENTRY_UUID_ORPHAN), Long.class);
		assertEquals(1, documentEntryIds.size());
		long documentEntryId = documentEntryIds.get(0);

		// Act
		subject.deleteDocumentEntries(documentEntryIds);

		// Assert
		List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from documententries where id = %d", documentEntryId));
		assertEquals(0, entries.size());
	}

	@Test
	public void deleteAssociations_emptyList_noop() {
		// Arrange
		List<Long> associationIds = Arrays.asList();

		// Act
		subject.deleteAssociations(associationIds);

		// Assert
		List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from associations where entryuuid = '%s'", ASSOCIATION_ENTRY_UUID_0));
		assertEquals(1, entries.size());
	}
	@Test
	public void deleteAssociations_nonEmptyList_deletesEntry() {
		// Arrange
		List<Long> associationIds = new JdbcTemplate(dataSource).queryForList(String.format("select id from associations where entryuuid = '%s'", TestConstants.ASSOCIATION_ENTRY_UUID_1), Long.class);
		assertEquals(1, associationIds.size());
		long associationId = associationIds.get(0);

		// Act
		subject.deleteAssociations(associationIds);

		// Assert
		List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from associations where id = %d", associationId));
		assertEquals(0, entries.size());
	}

	@Test
	public void deleteAssociations_largeList() {
		// Arrange
		List<Long> associationIds = new ArrayList<>();
		for(long i = 1; i <= 10000L; i++) {
			associationIds.add(i);
		}
		List<Long> associationIdsExisting = new JdbcTemplate(dataSource).queryForList("select id from associations", Long.class);
		associationIds.addAll(associationIdsExisting);
		long oneExistingId = associationIdsExisting.get(associationIdsExisting.size()-1);
		associationIds.removeIf(n -> n == oneExistingId);// for at sikre, at ikke alt slettes men en der findes er tilbage

		// Act
		subject.deleteAssociations(associationIds);

		// Assert
		List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList("select * from associations");
		assertEquals(1, entries.size());
	}

	@Test
	public void deleteSubmissionSets_emptyList_noop() {
		// Arrange
		List<Long> submissionSetIds = Arrays.asList();

		// Act
		subject.deleteSubmissionSets(submissionSetIds);

		// Assert
		List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from submissionsets where entryuuid = '%s'", SUBMISSION_SET_ENTRY_UUID_0));
		assertEquals(1, entries.size());
	}

	@Test
	public void deleteSubmissionSets_nonEmptyList_deletesEntry() {
		// Arrange
		List<Long> submissionSetIds = new JdbcTemplate(dataSource).queryForList(String.format("select id from submissionsets where entryuuid = '%s'", TestConstants.SUBMISSION_SET_ENTRY_UUID_4), Long.class);
		assertEquals(1, submissionSetIds.size());
		long submissionSetId = submissionSetIds.get(0);

		// Act
		subject.deleteSubmissionSets(submissionSetIds);

		// Assert
		List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from submissionsets where id = %d", submissionSetId));
		assertEquals(0, entries.size());
	}

	@Test
	public void markSuccessfulRegistryDeletion_emptyList_noop() {
		// Arrange
		List<DocumentEntry> documentEntryUuids = Arrays.asList();

		// Act
		subject.markSuccessfulRegistryDeletion(documentEntryUuids);

		// Assert
		List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from deleted_documententries where entryuuid = '%s'", DOCUMENT_ENTRY_UUID_0));
		assertEquals(0, entries.size());
	}
	@Test
	public void markSuccessfulRegistryDeletion_nonEmptyList_insertsEntries() {
		// Arrange
		List<DocumentEntry> documentEntryUuids = Arrays.asList(
				new DocumentEntry(1L, DOCUMENT_ENTRY_UUID_0, "unique-id-0"),
				new DocumentEntry(2L, DOCUMENT_ENTRY_UUID_1, "unique-id-1")
				);
		List<Map<String, Object>> entriesExisiting = new JdbcTemplate(dataSource).queryForList(String.format("select * from deleted_documententries where entryuuid = '%s'", DOCUMENT_ENTRY_UUID_0));
		assertEquals(0, entriesExisiting.size());

		// Act
		subject.markSuccessfulRegistryDeletion(documentEntryUuids);

		// Assert
		List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from deleted_documententries where entryuuid = '%s'", DOCUMENT_ENTRY_UUID_0));
		assertEquals(1, entries.size());

		entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from deleted_documententries where entryuuid = '%s'", DOCUMENT_ENTRY_UUID_1));
		assertEquals(1, entries.size());
	}

	@Test
	public void getDocumentEntriesForRepositoryDeletion_fetchesDataDeletedFromRegistry() {
		// Act
		List<DocumentEntry> result = subject.getDocumentEntriesForRepositoryDeletion(LIMIT);

		// Assert
		assertEquals(3, result.size());

		assertExpectedDocumentEntryExists(result, DELETED_DOCUMENT_ENTRY_UUID_1);
		assertExpectedDocumentEntryExists(result, DELETED_DOCUMENT_ENTRY_UUID_2);
		assertExpectedDocumentEntryExists(result, DELETED_DOCUMENT_ENTRY_UUID_3);
	}

	@Test
	public void markSuccessfulRepositoryDeletion_emptyList_noop() {
		// Arrange
		List<String> documentEntryUuids = Arrays.asList();
		List<Map<String, Object>> entriesExisiting = new JdbcTemplate(dataSource).queryForList("select * from deleted_documententries");
		assertEquals(3, entriesExisiting.size());

		// Act
		subject.markSuccessfulRepositoryDeletion(documentEntryUuids);

		// Assert
		List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList("select * from deleted_documententries");
		assertEquals("All entries should still exist in table", 3, entries.size());
	}

	@Test
	public void markSuccessfulRepositoryDeletion_DeletesFromTable() {
		// Arrange
		List<String> documentEntryUuids = Arrays.asList(DELETED_DOCUMENT_ENTRY_UUID_3);
		List<Map<String, Object>> entriesExisiting = new JdbcTemplate(dataSource).queryForList(String.format("select * from deleted_documententries where entryuuid = '%s'", DELETED_DOCUMENT_ENTRY_UUID_3));
		assertEquals(1, entriesExisiting.size());

		// Act
		subject.markSuccessfulRepositoryDeletion(documentEntryUuids);

		// Assert
		List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from deleted_documententries where entryuuid = '%s'", DELETED_DOCUMENT_ENTRY_UUID_3));
		assertEquals(0, entries.size());
	}

	@Test
	public void markSuccessfulRepositoryDeletion_WithUnknownUuid_DoesNothing() {
		// Arrange
		List<Map<String, Object>> before = new JdbcTemplate(dataSource).queryForList("select * from deleted_documententries");
		List<String> documentEntryUuids = Arrays.asList("unknown");

		// Act
		subject.markSuccessfulRepositoryDeletion(documentEntryUuids);

		// Assert
		List<Map<String, Object>> after = new JdbcTemplate(dataSource).queryForList("select * from deleted_documententries");
		assertEquals(before, after);
	}

	@Test
	public void markFailedRepositoryDeletion_emptyList_noop() {
		// Arrange
		List<String> documentEntryUuids = Arrays.asList();

		// Act
		subject.markFailedRepositoryDeletion(documentEntryUuids);

		// Assert
		List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from deleted_documententries where entryuuid = '%s' and deletion_status = '%s'", DELETED_DOCUMENT_ENTRY_UUID_3, DeletionStatus.DELETION_FROM_REPOSITORY_FAILED.toString()));
		assertEquals(0, entries.size());
	}

	@Test
	public void markFailedRepositoryDeletion_firstDeletionAttempt_updatesStatusAndIncrementsCounter() {
		// Arrange
		List<String> documentEntryUuids = Arrays.asList(DELETED_DOCUMENT_ENTRY_UUID_3);
		List<Map<String, Object>> entriesExisting = new JdbcTemplate(dataSource).queryForList(String.format("select * from deleted_documententries where deletion_status = '%s'", DeletionStatus.DELETION_FROM_REPOSITORY_FAILED.toString()));
		assertEquals(2, entriesExisting.size());

		// Act
		subject.markFailedRepositoryDeletion(documentEntryUuids);

		// Assert
		List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from deleted_documententries where deletion_status = '%s'", DeletionStatus.DELETION_FROM_REPOSITORY_FAILED.toString()));
		assertEquals(3, entries.size());

		Map<String, Integer> foundEntryUuids = new HashMap<>();
		foundEntryUuids.put((String) entries.get(0).get("entryuuid"), (Integer) entries.get(0).get("deletion_attempts"));
		foundEntryUuids.put((String) entries.get(1).get("entryuuid"), (Integer) entries.get(1).get("deletion_attempts"));
		foundEntryUuids.put((String) entries.get(2).get("entryuuid"), (Integer) entries.get(2).get("deletion_attempts"));

		assertTrue(foundEntryUuids.containsKey(DELETED_DOCUMENT_ENTRY_UUID_3));
		assertEquals(new Integer(1), foundEntryUuids.get(DELETED_DOCUMENT_ENTRY_UUID_3));
	}

	@Test
	public void markFailedRepositoryDeletion_secondDeletionAttempt_updatesStatus() {
		// Arrange
		List<String> documentEntryUuids = Arrays.asList(DELETED_DOCUMENT_ENTRY_UUID_2);
		List<Map<String, Object>> entriesExisting = new JdbcTemplate(dataSource).queryForList(String.format("select * from deleted_documententries where deletion_status = '%s'", DeletionStatus.DELETION_FROM_REPOSITORY_FAILED.toString()));
		assertEquals(2, entriesExisting.size());

		// Act
		subject.markFailedRepositoryDeletion(documentEntryUuids);

		// Assert
		List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from deleted_documententries where deletion_status = '%s'", DeletionStatus.DELETION_FROM_REPOSITORY_FAILED.toString()));
		assertEquals(2, entries.size());
		Map<String, Integer> foundEntryUuids = new HashMap<>();
		foundEntryUuids.put((String) entries.get(0).get("entryuuid"), (Integer) entries.get(0).get("deletion_attempts"));
		foundEntryUuids.put((String) entries.get(1).get("entryuuid"), (Integer) entries.get(1).get("deletion_attempts"));
		assertTrue(foundEntryUuids.containsKey(DELETED_DOCUMENT_ENTRY_UUID_2));
		assertEquals(new Integer(2), foundEntryUuids.get(DELETED_DOCUMENT_ENTRY_UUID_2));
	}

	@Test(expected = IllegalStateException.class)
	public void markFailedRepositoryDeletion_unknownUuid_throwsException() {
		// Arrange
		List<String> documentEntryUuids = Arrays.asList("some-unknown-id");

		// Act
		subject.markFailedRepositoryDeletion(documentEntryUuids);

		// Assert
	}

	@Test
	public void getHealthInfo_databaseOk_200() {
		// Arrange

		// Act
		HealthIndication health = subject.getHealthInfo();

		// Assert
		assertEquals(200, health.getStatus());
		assertTrue(health.getText(), health.getText().contains("OK: 3 entries currently enqueued for repository deletion."));
	}

	@Test
	public void getHealthInfo_failedEntriesExist_500() {
		// Arrange
		new NxrgTestDataCreator(dataSource).insertDeletedDocumentEntry(DELETED_DOCUMENT_ENTRY_UUID_4, DeletionStatus.DELETION_FROM_REPOSITORY_FAILED, 27);

		// Act
		HealthIndication health = subject.getHealthInfo();

		// Assert
		assertEquals(500, health.getStatus());
		assertTrue(health.getText(), health.getText().contains("ERROR: 1 entries could not be deleted from repository (at least " + DELETION_MAX_ATTEMPTS + " attempts)."));
	}

	@Test
	public void getDocumentEntryPatientIdsWithPrefix_WithNoMatchingPatientIds_ReturnsEmptyList() {
		List<String> patientIds = subject.getDocumentEntryPatientIdsWithPrefix("99");

		assertEquals(0, patientIds.size());
	}

	@Test
	public void getDocumentEntryPatientIdsWithPrefix_WithOneMatchingPatientId_ReturnsPatientId() {
		List<String> patientIds = subject.getDocumentEntryPatientIdsWithPrefix("13");

		assertEquals(Collections.singletonList(CPR_1), patientIds);
	}

	@Test
	public void getDocumentEntryPatientIdsWithPrefix_WithTwoMatchingPatientId_ReturnsPatientIds() {
		List<String> patientIds = subject.getDocumentEntryPatientIdsWithPrefix("14");

		assertEquals(2, patientIds.size());
		assertTrue(patientIds.contains(CPR_2));
		assertTrue(patientIds.contains(CPR_3));
	}

	@Test
	public void getDeprecatedDocumentsForDeletion_canFilterOnTypeCode_positive() {
		// Arrange
		TypeCode typeCode = TYPE_CODE_2;

		// Act
		List<DocumentEntry> result = subject.getDeprecatedDocumentsForDeletion("0101", typeCode,  60, LIMIT);

		// Assert
		assertEquals(2, result.size());
		assertEquals(TestConstants.DOCUMENT_ENTRY_UUID_9, result.get(0).getEntryUuid());

	}

	@Test
	public void getDeprecatedDocumentsForDeletion_canFilterOnTypeCode_negative() {
		// Arrange
		TypeCode typeCode = new TypeCode("Bogus", "Code");

		// Act
		List<DocumentEntry> result = subject.getDeprecatedDocumentsForDeletion("0101", typeCode,  60, LIMIT);

		// Assert
		assertEquals(0, result.size());

	}

	@Test
	public void getDeprecatedDocumentsForDeletion_canFilterOnDays() {
		// Arrange
		TypeCode typeCode = TYPE_CODE_2;

		// Act
		List<DocumentEntry> result = subject.getDeprecatedDocumentsForDeletion("0101", typeCode,  62, LIMIT);

		// Assert
		assertEquals(0, result.size());

	}

	@Test
	public void getDanglingSubmissionSetsForDeletion() {

		// Act
		List<SubmissionSet> result = subject.getDanglingSubmissionSetsForDeletion(LIMIT);

		// Assert
		assertEquals(2, result.size());
		result.sort((a, b) -> a.getEntryUuid().compareTo(b.getEntryUuid()));
		assertEquals(TestConstants.SUBMISSION_SET_ENTRY_UUID_4, result.get(0).getEntryUuid());
		assertEquals(TestConstants.SUBMISSION_SET_ENTRY_UUID_ORPHAN, result.get(1).getEntryUuid());

	}

	private void assertExpectedAssociationExists(List<Association> result, String entryUuid, String sourceUuid, String targetUuid) {
		assertTrue(result.stream().anyMatch(a ->
		a.getEntryUuid().equals(entryUuid) &&
		a.getSourceUuid().equals(sourceUuid) &&
		a.getTargetUuid().equals(targetUuid)));
	}

	private void assertExpectedDocumentEntryExists(List<DocumentEntry> result, String entryUuid) {
		assertTrue(result.stream().anyMatch(de -> de.getEntryUuid().equals(entryUuid)));
	}

	private void assertExpectedSubmissionSetExists(List<SubmissionSet> result, String entryUuid) {
		assertTrue(result.stream().anyMatch(ss -> ss.getEntryUuid().equals(entryUuid)));
	}
}
