package dk.nsp.xdscleanup.nxrg.service;

import dk.nsp.xdscleanup.common.model.Association;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.common.model.SubmissionSet;
import dk.nsp.xdscleanup.common.model.xds.AssociationType;
import dk.nsp.xdscleanup.common.model.xds.ReferenceType;
import dk.nsp.xdscleanup.common.model.xds.TypeCode;
import dk.nsp.xdscleanup.nxrg.dao.NxrgDao;
import dk.nsp.xdscleanup.sdm.dao.SdmDao;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import java.time.Instant;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static dk.nsp.xdscleanup.nxrg.dao.TestConstants.*;
import static org.junit.Assert.assertEquals;

@RunWith(MockitoJUnitRunner.class)
public class RegistryDeletionServiceTest {

    private final int LIMIT = 1000;
    @InjectMocks
    private RegistryDeletionService subject;

    @Mock
    private NxrgDao nxrgDao;

    @Mock
    private SdmDao sdmDao;

    private static final Association ASSOCIATION_0 = new Association(ASSOCIATION_ID_0, ASSOCIATION_ENTRY_UUID_0, AssociationType.HAS_MEMBER, SUBMISSION_SET_ENTRY_UUID_0, ReferenceType.SUBMISSION_SET, DOCUMENT_ENTRY_UUID_0, ReferenceType.DOCUMENT_ENTRY);
    private static final Association ASSOCIATION_1 = new Association(ASSOCIATION_ID_1, ASSOCIATION_ENTRY_UUID_1, AssociationType.HAS_MEMBER, SUBMISSION_SET_ENTRY_UUID_1, ReferenceType.SUBMISSION_SET, DOCUMENT_ENTRY_UUID_1, ReferenceType.DOCUMENT_ENTRY);
    private static final Association ASSOCIATION_2 = new Association(ASSOCIATION_ID_2, ASSOCIATION_ENTRY_UUID_2, AssociationType.HAS_MEMBER, SUBMISSION_SET_ENTRY_UUID_2, ReferenceType.SUBMISSION_SET, DOCUMENT_ENTRY_UUID_2, ReferenceType.DOCUMENT_ENTRY);

    private static final DocumentEntry DOCUMENT_ENTRY_0 = new DocumentEntry(DOCUMENT_ENTRY_ID_0, DOCUMENT_ENTRY_UUID_0, DOCUMENT_UNIQUE_ID_0);
    private static final DocumentEntry DOCUMENT_ENTRY_1 = new DocumentEntry(DOCUMENT_ENTRY_ID_1, DOCUMENT_ENTRY_UUID_1, DOCUMENT_UNIQUE_ID_1);
    private static final DocumentEntry DOCUMENT_ENTRY_2 = new DocumentEntry(DOCUMENT_ENTRY_ID_2, DOCUMENT_ENTRY_UUID_2, DOCUMENT_UNIQUE_ID_2);

    private static final SubmissionSet SUBMISSION_SET_0 = new SubmissionSet(SUBMISSION_SET_ID_0, SUBMISSION_SET_ENTRY_UUID_0);
    private static final SubmissionSet SUBMISSION_SET_1 = new SubmissionSet(SUBMISSION_SET_ID_1, SUBMISSION_SET_ENTRY_UUID_1);
    private static final SubmissionSet SUBMISSION_SET_2 = new SubmissionSet(SUBMISSION_SET_ID_2, SUBMISSION_SET_ENTRY_UUID_2);

    private static final DocumentEntry DELETED_DOCUMENT_ENTRY_0 = new DocumentEntry(DOCUMENT_ENTRY_ID_0, DOCUMENT_ENTRY_UUID_0, DOCUMENT_UNIQUE_ID_0);
    private static final DocumentEntry DELETED_DOCUMENT_ENTRY_1 = new DocumentEntry(DOCUMENT_ENTRY_ID_1, DOCUMENT_ENTRY_UUID_1, DOCUMENT_UNIQUE_ID_1);
    private static final DocumentEntry DELETED_DOCUMENT_ENTRY_2 = new DocumentEntry(DOCUMENT_ENTRY_ID_2, DOCUMENT_ENTRY_UUID_2, DOCUMENT_UNIQUE_ID_2);

    @Test
    public void markDeletionComplete_emptyList_noop() {
        // Arrange
        List<DocumentEntry> deletedDocumentEntries = Arrays.asList();

        // Act
        subject.markDeletionComplete(deletedDocumentEntries);

        // Assert
        Mockito.verify(nxrgDao, Mockito.never()).markSuccessfulRepositoryDeletion(Mockito.anyList());
    }

    @Test
    public void markDeletionComplete_nonEmptyList_marksDeletion() {
        // Arrange
        List<DocumentEntry> deletedDocumentEntries = Arrays.asList(DELETED_DOCUMENT_ENTRY_0, DELETED_DOCUMENT_ENTRY_1, DELETED_DOCUMENT_ENTRY_2);

        // Act
        subject.markDeletionComplete(deletedDocumentEntries);

        // Assert
        Mockito.verify(nxrgDao).markSuccessfulRepositoryDeletion(Arrays.asList(DELETED_DOCUMENT_ENTRY_0.getEntryUuid(), DELETED_DOCUMENT_ENTRY_1.getEntryUuid(), DELETED_DOCUMENT_ENTRY_2.getEntryUuid()));
    }

    @Test
    public void markDeletionAttemptFailed_emptyList_noop() {
        // Arrange
        List<DocumentEntry> deletedDocumentEntries = Arrays.asList();

        // Act
        subject.markDeletionAttemptFailed(deletedDocumentEntries);

        // Assert
        Mockito.verify(nxrgDao, Mockito.never()).markFailedRepositoryDeletion(Mockito.anyList());
    }

    @Test
    public void markDeletionAttemptFailed_nonEmptyList_marksFailedAttempt() {
        // Arrange
        List<DocumentEntry> deletedDocumentEntries = Arrays.asList(DELETED_DOCUMENT_ENTRY_0, DELETED_DOCUMENT_ENTRY_1, DELETED_DOCUMENT_ENTRY_2);

        // Act
        subject.markDeletionAttemptFailed(deletedDocumentEntries);

        // Assert
        Mockito.verify(nxrgDao).markFailedRepositoryDeletion(Arrays.asList(DELETED_DOCUMENT_ENTRY_0.getEntryUuid(), DELETED_DOCUMENT_ENTRY_1.getEntryUuid(), DELETED_DOCUMENT_ENTRY_2.getEntryUuid()));
    }

    @Test
    public void deleteDocumentEntries_deletesDocumentsAndRelatedObjects() {
        // Arrange
        Mockito.when(nxrgDao.getAssociationsForDocumentEntries(Arrays.asList(DOCUMENT_ENTRY_0.getEntryUuid(), DOCUMENT_ENTRY_1.getEntryUuid(), DOCUMENT_ENTRY_2.getEntryUuid())))
                .thenReturn(Arrays.asList(ASSOCIATION_0, ASSOCIATION_1, ASSOCIATION_2));
        Mockito.when(nxrgDao.determineSubmissionSetsLeftEmpty(Arrays.asList(SUBMISSION_SET_0.getEntryUuid(), SUBMISSION_SET_1.getEntryUuid(), SUBMISSION_SET_2.getEntryUuid()), Arrays.asList(DOCUMENT_ENTRY_0.getEntryUuid(), DOCUMENT_ENTRY_1.getEntryUuid(), DOCUMENT_ENTRY_2.getEntryUuid())))
                .thenReturn(Arrays.asList(SUBMISSION_SET_0, SUBMISSION_SET_1, SUBMISSION_SET_2));

        // Act
        subject.deleteDocumentEntries(Arrays.asList(DOCUMENT_ENTRY_0, DOCUMENT_ENTRY_1, DOCUMENT_ENTRY_2));

        // Assert
        Mockito.verify(nxrgDao).deleteAssociations(Arrays.asList(ASSOCIATION_0.getEntryId(), ASSOCIATION_1.getEntryId(), ASSOCIATION_2.getEntryId()));
        Mockito.verify(nxrgDao).deleteDocumentEntries(Arrays.asList(DOCUMENT_ENTRY_0.getEntryId(), DOCUMENT_ENTRY_1.getEntryId(), DOCUMENT_ENTRY_2.getEntryId()));
        Mockito.verify(nxrgDao).deleteSubmissionSets(Arrays.asList(SUBMISSION_SET_0.getEntryId(), SUBMISSION_SET_1.getEntryId(), SUBMISSION_SET_2.getEntryId()));
    }

    @Test
    public void deleteDocumentEntries_marksSuccessfulDeletion() {
        // Act
        subject.deleteDocumentEntries(Arrays.asList(DOCUMENT_ENTRY_0, DOCUMENT_ENTRY_1, DOCUMENT_ENTRY_2));

        // Assert
        Mockito.verify(nxrgDao).markSuccessfulRegistryDeletion(Arrays.asList(DOCUMENT_ENTRY_0, DOCUMENT_ENTRY_1, DOCUMENT_ENTRY_2));
    }

    @Test
    public void getDocumentEntriesForDeletion_GetsEntriesFromDao() {
        // Arrange
        TypeCode typeCode = new TypeCode("foo", "bar");
        Instant latestStartTime = Instant.now();
        List<DocumentEntry> entriesFromDao = Arrays.asList(new DocumentEntry(1, "", ""));
        Mockito.when(nxrgDao.getDocumentEntriesForDeletion(typeCode, latestStartTime, LIMIT))
                .thenReturn(entriesFromDao);

        // Act
        List<DocumentEntry> documentEntriesForDeletion = subject.getDocumentEntriesForDeletion(typeCode, latestStartTime, LIMIT);

        // Assert
        assertEquals(entriesFromDao, documentEntriesForDeletion);
    }

    @Test
    public void getDocumentEntriesForCitizens_GetsFromDao() {
        // Arrange
        List<String> cprs = Arrays.asList("1", "2", "3");
        List<DocumentEntry> documentEntriesFromDao = Collections.singletonList(Mockito.mock(DocumentEntry.class));
        Mockito.when(nxrgDao.getDocumentEntriesForCitizens(cprs))
                .thenReturn(documentEntriesFromDao);

        // Act
        List<DocumentEntry> documentEntries = subject.getDocumentEntriesForCitizens(cprs);

        // Assert
        assertEquals(documentEntriesFromDao, documentEntries);
    }

    @Test
    public void getDocumentEntryPatientIdsWithPrefix_GetsFromDao() {
        // Arrange
        String prefix = "04";
        List<String> patientIdsFromDao = Arrays.asList("01", "02", "03");
        Mockito.when(nxrgDao.getDocumentEntryPatientIdsWithPrefix(prefix)).thenReturn(patientIdsFromDao);

        // Act
        List<String> patientIds = subject.getDocumentEntryPatientIdsWithPrefix(prefix);

        // Assert
        assertEquals(patientIdsFromDao, patientIds);
    }

    @Test
    public void findDeletedDocumentEntries_GetsFromDao() {
        // Arrange
        List<DocumentEntry> deletedDocumentEntriesFromDao = Collections.singletonList(Mockito.mock(DocumentEntry.class));
        Mockito.when(nxrgDao.getDocumentEntriesForRepositoryDeletion(LIMIT)).thenReturn(deletedDocumentEntriesFromDao);

        // Act
        List<DocumentEntry> deletedDocumentEntries = subject.findDeletedDocumentEntries(LIMIT);

        // Assert
        assertEquals(deletedDocumentEntriesFromDao, deletedDocumentEntries);
    }
}