package dk.nsp.xdscleanup.openxds.dao.impl;

import dk.nsp.xdscleanup.common.model.Association;
import dk.nsp.xdscleanup.openxds.dao.OpenXdsDao;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class OpenXdsDaoImpl implements OpenXdsDao {
    private DataSource dataSource;

    private int inClauseMaxSize;

    public OpenXdsDaoImpl(DataSource dataSource, int inClauseMaxSize) {
        this.dataSource = dataSource;
        this.inClauseMaxSize = inClauseMaxSize;
    }

    @Transactional(propagation = Propagation.MANDATORY, transactionManager = "tx-openxds")
    public List<String> findExistingDocumentUniqueIds(List<String> documentUniqueIds) {
        if(documentUniqueIds == null || documentUniqueIds.isEmpty()) {
            return Arrays.asList();
        }

        List<String> existingDocumentUniqueIds = new ArrayList<>();

        // The list may be long, so we split it into chunks.
        for(int i = 0; i < documentUniqueIds.size(); i += inClauseMaxSize) {
            existingDocumentUniqueIds.addAll(findExistingDocumentUniqueIdsBatch(documentUniqueIds.subList(i, Math.min(i + inClauseMaxSize, documentUniqueIds.size()))));
        }

        return existingDocumentUniqueIds;
    }

    @Transactional(propagation = Propagation.MANDATORY, transactionManager = "tx-openxds")
    public void deleteDocuments(List<String> documentUniqueIds) {
        if(documentUniqueIds == null || documentUniqueIds.isEmpty()) {
            return;
        }

        // The list may be long, so we split it into chunks.
        for(int i = 0; i < documentUniqueIds.size(); i += inClauseMaxSize) {
            deleteDocumentsBatch(documentUniqueIds.subList(i, Math.min(i + inClauseMaxSize, documentUniqueIds.size())));
        }
    }

    private List<String> findExistingDocumentUniqueIdsBatch(List<String> documentUniqueIds) {
        String query = "select documentuniqueid from repository where documentuniqueid in (:documentUniqueIds)";

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("documentUniqueIds", documentUniqueIds);

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        return template.query(query, parameters, (rs, rowNum) -> rs.getString("documentuniqueid"));
    }

    private void deleteDocumentsBatch(List<String> documentUniqueIds) {
        String command = "delete from repository where documentuniqueid in (:documentUniqueIds)";

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("documentUniqueIds", documentUniqueIds);

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        template.update(command, parameters);
    }
}
