package dk.nsp.xdscleanup.openxds.dao.impl;

import dk.nsp.xdscleanup.openxds.TestDataCreator;
import dk.nsp.xdscleanup.openxds.setup.EmbeddedDatabaseSetup;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import javax.sql.DataSource;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import static dk.nsp.xdscleanup.openxds.TestConstants.*;
import static org.junit.Assert.assertEquals;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = {EmbeddedDatabaseSetup.class})
public class OpenXdsDaoImplTest {
    private OpenXdsDaoImpl subject;

    private static final int IN_CLAUSE_MAX_SIZE = 1;

    @Autowired
    private DataSource dataSource;

    @Before
    public void setup() {
        new TestDataCreator(dataSource).initializeTestDatabase();

        subject = new OpenXdsDaoImpl(dataSource, IN_CLAUSE_MAX_SIZE);
    }

    @Test
    public void findExistingDocumentUniqueIds_emptyList_noop() {
        // Arrange
        List<String> documentUniqueIds = Arrays.asList();

        // Act
        List<String> result = subject.findExistingDocumentUniqueIds(documentUniqueIds);

        // Assert
        assertEquals(0, result.size());
    }

    @Test
    public void findExistingDocumentUniqueIds_includesFoundIds() {
        // Arrange
        List<String> documentUniqueIds = Arrays.asList(DOCUMENT_UNIQUE_ID_0, DOCUMENT_UNIQUE_ID_1);

        // Act
        List<String> result = subject.findExistingDocumentUniqueIds(documentUniqueIds);

        // Assert
        assertEquals(2, result.size());
        assertEquals(DOCUMENT_UNIQUE_ID_0, result.get(0));
        assertEquals(DOCUMENT_UNIQUE_ID_1, result.get(1));
    }

    @Test
    public void findExistingDocumentUniqueIds_excludesMissingIds() {
        // Arrange
        List<String> documentUniqueIds = Arrays.asList(DOCUMENT_UNIQUE_ID_NONEXISTENT);

        // Act
        List<String> result = subject.findExistingDocumentUniqueIds(documentUniqueIds);

        // Assert
        assertEquals(0, result.size());
    }

    @Test
    public void deleteDocuments_emptyList_noop() {
        // Arrange
        List<String> documentUniqueIds = Arrays.asList();

        // Act
        subject.deleteDocuments(documentUniqueIds);

        // Assert
        List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from repository where documentuniqueid in ('%s', '%s')", DOCUMENT_UNIQUE_ID_0, DOCUMENT_UNIQUE_ID_1));
        assertEquals(2, entries.size());
        assertEquals(DOCUMENT_UNIQUE_ID_0, entries.get(0).get("documentuniqueid"));
        assertEquals(DOCUMENT_UNIQUE_ID_1, entries.get(1).get("documentuniqueid"));
    }

    @Test
    public void deleteDocuments_deletesDocumentsFromList() {
        // Arrange
        List<String> documentUniqueIds = Arrays.asList(DOCUMENT_UNIQUE_ID_0, DOCUMENT_UNIQUE_ID_1);

        // Act
        subject.deleteDocuments(documentUniqueIds);

        // Assert
        List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from repository where documentuniqueid in ('%s', '%s', '%s', '%s')", DOCUMENT_UNIQUE_ID_0, DOCUMENT_UNIQUE_ID_1, DOCUMENT_UNIQUE_ID_2, DOCUMENT_UNIQUE_ID_3));
        assertEquals(2, entries.size());
        assertEquals(DOCUMENT_UNIQUE_ID_2, entries.get(0).get("documentuniqueid"));
        assertEquals(DOCUMENT_UNIQUE_ID_3, entries.get(1).get("documentuniqueid"));
    }

    @Test
    public void deleteDocuments_ignoresUnknownDocuments() {
        // Arrange
        List<String> documentUniqueIds = Arrays.asList(DOCUMENT_UNIQUE_ID_NONEXISTENT);

        // Act
        subject.deleteDocuments(documentUniqueIds);

        // Assert
        List<Map<String, Object>> entries = new JdbcTemplate(dataSource).queryForList(String.format("select * from repository where documentuniqueid in ('%s', '%s', '%s', '%s')", DOCUMENT_UNIQUE_ID_0, DOCUMENT_UNIQUE_ID_1, DOCUMENT_UNIQUE_ID_2, DOCUMENT_UNIQUE_ID_3));
        assertEquals(4, entries.size());
        assertEquals(DOCUMENT_UNIQUE_ID_0, entries.get(0).get("documentuniqueid"));
        assertEquals(DOCUMENT_UNIQUE_ID_1, entries.get(1).get("documentuniqueid"));
        assertEquals(DOCUMENT_UNIQUE_ID_2, entries.get(2).get("documentuniqueid"));
        assertEquals(DOCUMENT_UNIQUE_ID_3, entries.get(3).get("documentuniqueid"));
    }
}